**************************************************************************;
** Macro name      : CalculateKnotsPartition                            **;
** Version number  : 1.01                                               **;
** Authors         : Alberto Garcia-Hernandez                           **;
**                   Dimitris Rizopoulos                                **;
** Date            : 15-Dec-2014                                        **;
** SAS version     : 9.0 or higher                                      **;
** Description     : Given a variable creates a partition so the        **; 
**                   amount of observations is equally distributed      **;
** Parameters      : indata       = Input Dataset                       **;
**                   outdata      = Output Dataset                      **;
**                   var          = Input Variable of interest          **;
**                   number       = Number of knots to create           **;
**                   knotname     = Output variable suffix              **;
**                   wh           = Condition to restrict the data used **;
**                                                                      **;
**                                                                      **;
**                        MACRO HISTORY                                 **;
** Version     Date        Changes                                      **;
** 1.01        15/12/2014  First Production Version                     **;
**                                                                      **;
** Copyright (C) 2014      Alberto Garcia Hernandez                     **;
**                         Dimitris Rizopoulos                          **;
** This program is free software: you can redistribute it and/or modify **;
** it under the terms of the GNU General Public License as published by **;
** the Free Software Foundation, either version 3 of the License, or    **;
** (at your option) any later version.                                  **;
**                                                                      **;
** This program is distributed in the hope that it will be useful,      **;
** but WITHOUT ANY WARRANTY, without even the implied warranty of       **;
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **;
** GNU General Public License for more details.                         **;
**                                                                      **;
** You should have received a copy of the GNU General Public License    **;
** along with this program.  If not, see <http://www.gnu.org/licenses/  **;
** Everyone is permitted to copy and distribute verbatim copies         **;
** of this program, but changing it is not allowed.                     **;
**                                                                      **;
** Contact email: alberto.garciahernandez@astellas.com                  **;
**************************************************************************;

%macro calculateknotspartition(indata=, outdata=, var=, Number=, knotname=, wh=);  
%if &Number > 0  %then %do;
  %if &&&knotname.&Number. =  %then %do;
  %let listpct1 = %sysevalf(100*(1/(&Number+1)));
  %let listpct2 = 1;
  %let listknots = :&knotname.1;

  %do index = 2 %to &Number ;
    %let listpct1 = &listpct1.   %sysevalf(100*(&index./ (&Number+1)));
	%let listpct2 = &listpct2.  &index. ;
	%let listknots = &listknots. , :&knotname.&index.;
  %end;
  
%put wh is &wh. ;
  proc univariate data=&indata.  noprint;
  var  &var.;
  output out=&outdata. pctlpts = &listpct1. pctlpre=pct  ;
  %if %NRQUOTE(&wh.) NE %then where &wh.;;
  run;
 
  proc sql noprint;
  select compress("put("!!name!!", hex16.)") into:vlist separated by ','
  from dictionary.columns where memname=upcase("&outdata.");
  quit;

  proc sql noprint;
   select &vlist. into &listknots.
   from &outdata.;
  quit;

 %end;
%end;
   
%mend calculateknotspartition;
