**************************************************************************;
** Macro name      : JM                                                 **;
** Version number  : 02.01                                              **;
** Authors         : Alberto Garcia-Hernandez                           **;
**                   Dimitris Rizopoulos                                **;
** Date            : 10-September2015                                   **;
** SAS version     : 9.0 or higher                                      **;
** Description     : JM is a  SAS macro that fits jointly               **;
**                   generalized mixed models for longitudinal data and **;
**                   proportional hazards models for time-to-event      **;
**                   responses                                          **; 
** Parameters      : See full list of parameters in the Reference Manual**;
** Macro history   : See in Reference Manual                            **;
**                                                                      **;
**                                                                      **;
** Copyright (C) 2014      Alberto Garcia Hernandez                     **;
**                         Dimitris Rizopoulos                          **;
**                                                                      **;
**                                                                      **;
** This program is free software: you can redistribute it and/or modify **;
** it under the terms of the GNU General Public License as published by **;
** the Free Software Foundation, version 3 of the License.              **;
**                                                                      **;
** This program is distributed in the hope that it will be useful,      **;
** but WITHOUT ANY WARRANTY, without even the implied warranty of       **;
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **;
** GNU General Public License for more details.                         **;
**                                                                      **;
** You should have received a copy of the GNU General Public License    **;
** along with this program.  If not, see <http://www.gnu.org/licenses/  **;
** Everyone is permitted to copy and distribute verbatim copies         **;
** of this program, but changing it is not allowed.                     **;
**                                                                      **;
** This macro is available at http://www.jm-macro.com                   **;
** Contact email: alberto.garciahernandez@astellas.com                  **;
**************************************************************************;                  

%macro jm(	Data 			= , 
			DataStructure   = ,
			Where 			= , 
			SubjectVar 		= ,  
			InitialParameters = ,
			OutputParameters = ,
            OutputPredictions = ,
			OutputPredictData = ,
			ListingFile=, 

		    LongiType 		= ,
			LongiLink 		= ,
			LongiVar 		= ,
			LongiEventCat 	= ,
			LongiTrialsVar 	= ,
			LongiOffset     = ,
			LongiTimeModel 	= ,
			LongiTimeVar 	= ,  
			LongiLowerknot 	= ,	
			LongiUpperknot 	= ,
			LongiNknots  	= , 
			LongiDegree  	= ,
			Longiknot1 		= , Longiknot2 = , Longiknot3 = , Longiknot4 = , Longiknot5 = , Longiknot6 = , Longiknot7 = , Longiknot8 = , Longiknot9 = , 
            LongiCovariates = , 
			LongiTimeInter 	= , 
			LongiGMatrix 	= ,
			LongiGMatrixParam = , 
			LongiSmallLogSD = , 
			LongiModelOptions = ,
			LongiGLINLOptions = ,

			EventTimeVar	= , 
			EventVar		= , 
			EventVal		= ,
			EventModel		= ,
			EventTimeStart  = ,
			EventStrata     = , 
			EventDegree     = ,
			EventLowerknot 	= ,	
			EventUpperknot 	= ,
			EventNknots 	= ,
			Eventknot1 = , Eventknot2 = , Eventknot3 = , Eventknot4 = , Eventknot5 = , Eventknot6 = , Eventknot7 = , Eventknot8 = , Eventknot9 = , 
			EventCovariates = , 
            SharedParam    	= ,
			SharedLongiTerm	= ,
			SharedLagging 	= ,	
			SharedParamInt 	=  ,
			SharedCoefficients = ,
			NLMixedOptions	= ,
            AdditionalOptions= ,
			SmallTimeDiff =,
            NLMixedExpression = ,
            Transform_Current_Value_Term = );


/*---Date and Time at exectution start---*/
data _null_;
  tempdate=put(date(),hex16.);
  temptime=put(time(),hex16.);
  call symput('startdate',tempdate);
  call symput('starttime',temptime);
run;
	
/*---change to uppercase---*/
%let Data     				= %upcase(&Data);
%let DataStructure			= %upcase(&DataStructure);
%let Where     			    = %upcase(&Where);
%let SubjectVar     		= %upcase(&SubjectVar);
%let LongiTimeModel		    = %upcase(&LongiTimeModel);
%let LongiCovariates		= %upcase(&LongiCovariates);
%let LongiTimeInter		    = %upcase(&LongiTimeInter);
%let LongiTimevar			= %upcase(&LongiTimevar);
%let LongiType				= %upcase(&LongiType);
%let LongiLink				= %upcase(&LongiLink);
%let LongiVar				= %upcase(&LongiVar);
%let LongiGMatrix			= %upcase(&LongiGMatrix);
%let LongiGMatrixParam		= %upcase(&LongiGMatrixParam);
%let EventTimeVar			= %upcase(&EventTimeVar);
%let EventVar				= %upcase(&EventVar);
%let EventVal				= %upcase(&EventVal);
%let EventModel			    = %upcase(&EventModel);
%let EventCovariates		= %upcase(&EventCovariates);
%let EventStrata		    = %upcase(&EventStrata);
%let SharedParam			= %upcase(&SharedParam);
%let SharedLongiTerm		= %upcase(&SharedLongiTerm);
%let SharedParamInt	     	= %upcase(&SharedParamInt);
%let SharedCoefficients	    = %upcase(&SharedCoefficients);
%let InitialParameters		= %upcase(&InitialParameters);
%let AdditionalOptions		= %upcase(&AdditionalOptions);
%let OutputParameters		= %upcase(&OutputParameters);
%let OutputPredictions		= %upcase(&OutputPredictions);
*Section below added in version 1.08 to avoid %SYSMACEXIST and make the macro functionable before SAS 9.3;
%let NLMixedExpression		= %upcase(&NLMixedExpression);
*Section below added in version 02.01 to allow transformations over m_i(t);
%let Transform_Current_Value_Term		= %upcase(&Transform_Current_Value_Term);

%macro elementinlist(element, list);
  %let counter=0;
  %do %while(%qscan(&list.,&counter+1,%str( )) ne %str());
    %let counter = %eval(&counter+1);
  %end;
  %let output=0;
  %do i = 1 %to &counter;
    %if &element = %scan(&list. ,&i, ' ') %then %let output=1;
  %end;
  &output.
%mend;

 *****    Giving default parameters in case not given or given as blanks*******;
%if &DataStructure. = %then %let DataStructure = STANDARD;
%if &LongiType. =  %then %let LongiType = NORMAL;
%if &LongiType. = NORMAL and &LongiLink. =  %then %let LongiLink = IDENTITY;
%if &LongiType. = NORMAL and "&LongiModelOptions." = "" %then %let LongiModelOptions = METHOD=ML;
%if (&LongiType. = BINARY or &LongiType. = BINARY2 or &LongiType. = BINOMIAL) and &LongiLink. =  %then %let LongiLink = LOGIT;
%if &LongiType. = POISSON and &LongiLink. =  %then %let LongiLink = LOG;
%if &LongiNknots. = %then %let LongiNknots  = 0;
%if &LongiTimeModel. = %then %let LongiTimeModel  = LINEAR;
%if &LongiDegree. = %then %let LongiDegree = 1;
%if "&LongiGMatrix." = "" %then %let LongiGMatrix = VC;
%if &LongiGMatrixParam. = %then %let LongiGMatrixParam = VARCOVAR;
%if &EventDegree. = %then %let EventDegree = 1;
%if &EventNknots. = %then %let  EventNknots = 6;
%if &SharedParam. = %then %let SharedParam = CURRENT_VALUE;
%if &SharedParam. = COEFFICIENTS %then %let SharedLongiTerm  = COEFFICIENTS;
%if &SharedLongiTerm. = %then %let SharedLongiTerm = MEAN;
%if &SharedLagging. = %then %let  SharedLagging = 0;
%if &InitialParameters. = %then %let  InitialParameters = DISJOINT;
%if &OutputParameters. = %then %let OutputParameters  = _JMOUTPUTPARAMETERS;
%if &SmallTimeDiff. = %then %let  SmallTimeDiff = 0.001;
%if &LongiSmallLogSD. = %then %let  LongiSmallLogSD = -10;
%if (&LongiType.= BINARY or &LongiType.= BINARY2) and &LongiEventCat. = %then %let  LongiEventCat = 1;
%if  %elementinlist(ONLYEVENT, &AdditionalOptions.) %then %let SharedParam = NONE;
%if  %elementinlist(ONLYLONGI, &AdditionalOptions.) %then %let SharedParam = NONE;
%let longi_nparam =;
%let event_nparam =;
*Section below added in version 1.08 to avoid %SYSMACEXIST and make the macro functionable before SAS 9.3;
%if &NLMixedExpression. = %then %let NLMixedExpression = NO;
*Section below added in version 02.01 to allow transformations over m_i(t);
%if &Transform_Current_Value_Term. = %then %let Transform_Current_Value_Term = NO;

****** Initial Checks. Program is aborted if unexpected input is found  *******; 

%if &SubjectVar. = %then %do;
%PUT %str(ER)ROR: SubjectVar cannot be left blank; 
   %ABORT;
%end;

%if &DataStructure. NE VERTICAL %then %let DataStructure = STANDARD;;

%if   NOT (%elementinlist(&LongiTimeModel.,LINEAR SPLINE BSPLINE NATURALCUBIC)) %then %do;
   %PUT %str(ER)ROR: Available models for the longitudinal response (LongiTimeModel) are: LINEAR SPLINE BSPLINE NATURALCUBIC; 
   %ABORT;  
%end;

%if   NOT (%elementinlist(&LongiType., NORMAL BINARY BINARY2 BINOMIAL POISSON)) %then %do;
   %PUT %str(ER)ROR: Available distributions for the longitudinal response (LongiType) are: NORMAL BINARY BINARY2 BINOMIAL POISSON; 
   %ABORT;  
%end;

%if not (%elementinlist("&LongiGMatrix.","VC" "UN" "ANTE(1)")) %then %do;
   %PUT %str(ER)ROR: Available G matrices are: VC UN ANTE(1); 
   %ABORT;  
%end;

%if NOT (%elementinlist(&LongiGMatrixParam,VARCOVAR CHOL VARCORR)) %then %do;
   %PUT %str(ER)ROR: Available G matrix parameterizations are: VARCOVAR CHOL VARCORR; 
   %ABORT;  
%end;

%if "&LongiGMatrix." = "ANTE(1)" and &LongiGMatrixParam. NE VARCORR %then %do;
   %PUT %str(ER)ROR: For the ANTE G matrix only the VARCORR parameterization is available. Please set LongiGMatrixParam=VARCORR; 
   %ABORT;  
%end;

%if (%elementinlist("&LongiGMatrix.","VC" "UN")) and &LongiGMatrixParam. = VARCORR %then %do;
   %PUT %str(ER)ROR: For the VC or UN G matrices only VARCOVAR or CHOL parameterizations are available.; 
   %ABORT;  
%end;


%if NOT (%elementinlist(&SharedLongiTerm., MEAN LINEAR_PREDICTOR COEFFICIENTS)) %then %do;
*Statements below modified in version 1.05 (COEFFICIENTS ADDED);
   %PUT %str(ER)ROR: Available shared parameter terms are: MEAN LINEAR_PREDICTOR COEFFICIENTS.; 
   %ABORT;  
%end;

%if  ((&SharedLongiTerm. NE LINEAR_PREDICTOR) and (&LongiLink. NE IDENTITY) and (%elementinlist(CUMULATIVE, &SharedParam.))) %then %do;
   %PUT %str(ER)ROR: The cumulative parameterization is only available either using the identity link function or using SharedLongiTerm = LINEAR_PREDICTOR; 
   %ABORT;  
%end;

%if &LongiType.= BINOMIAL and &LongiTrialsVar. = %then %do;
   %PUT %str(ER)ROR: Binomial distribution for the longitudinal response requires the specification of the number of trials (LongiTrialsVar); 
   %ABORT;
%end;

%if   NOT (%elementinlist(&EventModel., EXPONENTIAL WEIBULL PIECEWISE GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE)) %then %do;
   %PUT %str(ER)ROR: Available models for the event response (LongiTimeModel) are: EXPONENTIAL WEIBULL PIECEWISE GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE; 
   %ABORT;  
%end;

%if %sysfunc(exist(&Data))= 0 %then %do;
   %PUT %str(ER)ROR: &Data DATASET DOES NOT EXIST; 
   %ABORT;  
%end;
%if &EventModel. = PIECEWISE and (&EventNknots.<1 or &EventNknots.>9) %then %do; 
   %PUT %str(ER)ROR:  NUMBER ALLOWED OF KNOTS FOR THE PIECEWISE EXPONENTIAL MODEL IS 1-9;%ABORT; 
%end; 
%if (%elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE)) and (&EventNknots.<0 or &EventNknots.>9) %then %do; 
   %PUT %str(ER)ROR:  NUMBER ALLOWED OF KNOTS FOR SPLINES AND ROYSTON-PARMAR MODELS IS 0-9;%ABORT; 
%end;

%if &LongiNknots.>9 %then %do; 
   %PUT %str(ER)ROR: MAXIMUM NUMBER ALLOWED OF KNOTS FOR THE LONGITUDINAL MODEL IS 9;%ABORT; 
%end; 
%if %index(%upcase(&LongiCovariates),T_) %then %do; 
   %PUT %str(ER)ROR: TIME_ STRING NOT ALLOWED AS COVARIATE NAME;%ABORT;
%end; 
%if %index(%upcase(&LongiCovariates),SCALE) %then %do; 
   %PUT %str(ER)ROR: SCALE STRING NOT ALLOWED AS COVARIATE NAME;%ABORT;
%end; 
%if %index(%upcase(&LongiCovariates),SHAPE) %then %do; 
   %PUT %str(ER)ROR: SHAPE STRING NOT ALLOWED AS COVARIATE NAME; %ABORT;
%end; 
%if &OutputParameters. = &InitialParameters. %then %do; 
   %PUT %str(ER)ROR: INPUT PARAMETERS DATASET SHOULD BE DIFFERENT FROM THE OUTPUT PARAMETER DATASET;%ABORT;
%end; 
%if &InitialParameters. NE DISJOINT and not %sysfunc(exist(&InitialParameters.)) %then %do;
   %PUT %str(ER)ROR:  &InitialParameters. DATASET DOES NOT EXIST; %ABORT;
%end; 
%if (%elementinlist(COEFFICIENTS, &SharedParam.)) and &SharedCoefficients. = %then %do;
   %PUT %str(ER)ROR:  RANDOM EFFECTS PARAMETERZIATION REQUIRES RANDOM EFFECTS IN THE SharedCoefficients PARAMETER;%ABORT;
%end;

*Two new items added in V02.01 baseline_value and change_from_baseline;
%if NOT ((%elementinlist(CURRENT_VALUE, &SharedParam.)) or (%elementinlist(SLOPE , &SharedParam.)) or (%elementinlist(CUMULATIVE, &SharedParam.)) or (%elementinlist(COEFFICIENTS, &SharedParam.)) or (%elementinlist(NONE, &SharedParam.))
or (%elementinlist(BASELINE_VALUE, &SharedParam.)) or (%elementinlist(CHANGE_FROM_BASELINE, &SharedParam.)) 
)  %then %do;
   %PUT %str(ER)ROR:  UNEXPECTED PARAMETERZIATION IN THE SHAREDPARAM PARAMETER;%ABORT;
%end;
%if (%elementinlist(CUMULATIVE, &SharedParam.)) and &SharedLagging.>0 %then %do;
   %PUT %str(ER)ROR: THE MACRO DOES NOT SUPPORT CUMULATIVE SHARED PARAMTER WITH LAGGING EFFECTS;%ABORT;
%end;


*Two new items added in V02.01 baseline_value and change_from_baseline;
%let KronrodBasis = YES; 
%if NOT ((%elementinlist(CURRENT_VALUE, &SharedParam.)) or (%elementinlist(SLOPE, &SharedParam.)) or (%elementinlist(CUMULATIVE, &SharedParam.))
or (%elementinlist(CHANGE_FROM_BASELINE, &SharedParam.)) 
)  %then %do;
   %let KronrodBasis = NO; 
%end;

*Group of lines below was updated in Macro Version 1.03 ;
%let Number_AddOptions = 0;
 %do %while(%qscan(&AdditionalOptions.,&Number_AddOptions+1,%str( )) ne %str());
  %let Number_AddOptions = %eval(&Number_AddOptions+1);
  %if NOT %elementinlist(%scan(&AdditionalOptions. ,&Number_AddOptions.), CALCULATEEXECTIME COMPETING EVENTTIMES FITSTATISTICS ONLYEVENT ONLYLONGI NOINT NOMACROWARN SKIPMACROHEADER UNTOUCHEDLISTING)   %then %do;
    %PUT %str(ER)ROR:  UNEXPECTED ADDITIONAL OPTION. THE ALLOWED OPTIONS ARE: CALCULATEEXECTIME COMPETING EVENTTIMES FITSTATISTICS ONLYEVENT ONLYLONGI NOINT NOMACROWARN SKIPMACROHEADER UNTOUCHEDLISTING.;
    %ABORT;
  %end;
 %end;

 
****** Check that input variables exist*******; 
%macro VarExist(ds,var);
data _null_;
dsid=open("&ds");
if "&var" ne "" and varnum(dsid, "&var") = 0 then do;
       PUT "ER" "ROR:  VARIABLE &var. DOES NOT EXIST IN &ds. DATASET";
       ABORT;
end;
run;
%mend VarExist;
%if &DataStructure. = VERTICAL %then %VarExist(&data., JMDist);;
%VarExist(&data.,&SubjectVar.);
%VarExist(&data.,&LongiVar.);
%VarExist(&data.,&LongiTimevar.);
%if &LongiTrialsVar. NE %then %VarExist(&data.,&LongiTrialsVar.);;
%if &EventStrata. NE %then %VarExist(&data.,&EventStrata.);;
%VarExist(&data.,&EventTimeVar.);
%VarExist(&data.,&EventVar.);

*Section below included in version 1.07 ;
****** Check that input dataset does not have variables with names internally used by the macro*******; 
%macro ShouldNotExist(ds,var);
data _null_;
dsid=open("&ds");
if "&var" ne "" and varnum(dsid, "&var") > 0 then do;
       PUT "WAR" "NING:  VARIABLE &var. EXIST IN &ds. DATASET.";
       PUT "                THIS VARIABLE NAME IS USED INTERNALLY BY THE MACRO SO ORIGINAL VALUES WILL BE OVERWRITTEN. ";
	   PUT "                UNEXPEXTED RESULTS MAY OCCUR. AVOID VARIABLE NAMES STARTING WITH 'JM' IN &ds..";
       ABORT;
end;
run;
%mend ShouldNotExist;
%if &DataStructure. = STANDARD %then %ShouldNotExist(&data, JMDist);;
%ShouldNotExist(&data., JMTimeObs);
%ShouldNotExist(&data., JMResponse);
%ShouldNotExist(&data., JMFromTimeObs);
%ShouldNotExist(&data., JMEvent);
%ShouldNotExist(&data., JMLagTimeObs);
%ShouldNotExist(&data., JMStrata);
%ShouldNotExist(&data., JMStrataN);
%ShouldNotExist(&data., JMTimeObs_kr_1);
%ShouldNotExist(&data., JM_kr_weight1);
%ShouldNotExist(&data., JMTimeObs_b_0);
%ShouldNotExist(&data., JMLagTimeObs_kr_1);
%ShouldNotExist(&data., JMTimeObs_kr_b_0);


****** Convert Knots given by the user into Hex16. format *******; 
%if not (%elementinlist(ONLYEVENT, &AdditionalOptions)) %then  %do; 
%do i= 1 %to &LongiNknots.;
	%if &&Longiknot&i. NE %then %let Longiknot&i. = %sysfunc(putn(&&Longiknot&i. ,hex16.));;
%end;
%if &LongiLowerknot. NE %then %let LongiLowerknot = %sysfunc(putn(&LongiLowerknot.  ,hex16.));;
%if &LongiUpperknot. NE %then %let LongiUpperknot = %sysfunc(putn(&LongiUpperknot.  ,hex16.));;
%end;

%if not (%elementinlist(ONLYLONGI, &AdditionalOptions.) ) %then %do;
%if %elementinlist(&EventModel. , WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;
  %do i= 1 %to 9;
    %let LogEventknot&i. = ;
  %end;

  %do i= 1 %to &EventNknots.;
    %if &&Eventknot&i. NE %then %do;
        %let LogEventknot&i. = %sysfunc(log(&&Eventknot&i.));
        %let LogEventknot&i. = %sysfunc(putn(&&LogEventknot&i. ,hex16.));;
    %end;
  %end;
  %if &EventLowerknot. NE %then %do;
    %let LogEventLowerknot = %sysfunc(log(&EventLowerknot.));;
    %let LogEventLowerknot = %sysfunc(putn(&LogEventLowerknot.  ,hex16.));
  %end;

  %if &EventUpperknot. NE %then %do;
    %let LogEventUpperknot = %sysfunc(log(&EventUpperknot.));
	%let LogEventUpperknot = %sysfunc(putn(&LogEventUpperknot.  ,hex16.));
  %end;
%end;

%do i= 1 %to &EventNknots.;
	%if &&Eventknot&i. NE %then %let Eventknot&i. = %sysfunc(putn(&&Eventknot&i. ,hex16.));;
%end;
%if &EventLowerknot. NE %then %let EventLowerknot = %sysfunc(putn(&EventLowerknot.  ,hex16.));;
%if &EventUpperknot. NE %then %let EventUpperknot = %sysfunc(putn(&EventUpperknot.  ,hex16.));;

%end;

***** Removing work datasets in case they exist from previous runs;
***** This is done to avoid that an error during the execution of this run takes by mistake datasets from previous runs;
%macro removedataset(dataset);
 %if %sysfunc(exist(&dataset.)) %then %do; 
  proc datasets lib=work nolist; delete &dataset.;
  quit;run;
 %end;
%mend removedataset;
%removedataset(_JMData);
%removedataset(_JMSolutionF);
%removedataset(_JMSolutionF2);
%removedataset(_JMParameterestimates);
%removedataset(_JMParameterestimates2);
%removedataset(_JMSolutionf);
%removedataset(_JMSolutionf2);
%removedataset(_JMCovparms);
%removedataset(_JMCovparms2);
%removedataset(_JMCholg);
%removedataset(_JMCholg2);
%removedataset(_JMPieces);
%removedataset(_JMPieces2);
%removedataset(_JMInitialParametersNormalized);
%removedataset(_JMOutputParameters);
%removedataset(_JMFitStatistics);
%removedataset(_JMCalculated_Knots_Longi_Model);
%removedataset(_JMCalculated_Knots_Event_Model);
%removedataset(_JMDummyEventData);
%removedataset(_JMSplinesParam);
%removedataset(_JMSplinesParam2);


**** Closing the listing ouput, it will be opened again later to output only the most relevant info; 
%if NOT (%elementinlist(UNTOUCHEDLISTING, &AdditionalOptions.)) %then %do;
  ods listing close;
%end;

**** Creating the _JMData from the InputData - The _JMData Set will be used for the Analysis; 
data _JMData;
  set &Data.;
  if &SubjectVar. = ""  %if NOT (%elementinlist(NOMACROWARN, &AdditionalOptions.)) and %NRQUOTE(&where.) NE %then and &where.; then put "WAR" "NING: Entries with empty subject identifier will be assumed to be from the same subject."; 
  %if %NRQUOTE(&where.) NE %then if &where. then; output; 
run;

%if  &DataStructure. = STANDARD %then %do;

**** Sorting data by subject ID;
proc sort data=_JMData;
  by &SubjectVar. %if not (%elementinlist(ONLYEVENT, &AdditionalOptions.)) %then &LongiTimevar. ;;
run;

**** Transforming the _JMData dataset into a long (vertical structure); 

data _JMData;
  set _JMData;
  by &SubjectVar.  &LongiTimevar.;
  %if not (%elementinlist(ONLYEVENT, &AdditionalOptions.)) %then  %do;  
  if index("&AdditionalOptions.","NOMACROWARN") = 0 and &LongiTimevar. > &EventTimeVar. and &LongiVar. ne . then put "WAR" "NING: Subject " &SubjectVar. "has a longitudinal response AFTER the event. These observations are not removed.";
  JMDist = "LONGI";
  
  JMTimeObs = 	&LongiTimevar.;
  JMResponse = 		&LongiVar.;

  if &LongiTimevar. NE . then output;
  %end;
  %if not (%elementinlist(ONLYLONGI, &AdditionalOptions.)) %then  %do;
  

  if last.&SubjectVar. then do;
  JMDist = "EVENT";
  JMFromTimeObs=0; 
  JMResponse = 		&EventTimeVar.;
  JMTimeObs = 	    &EventTimeVar.;
  JMEvent= 			&EventVar. = "&EventVal.";
  *JMLagTimeObs are the times points at which we will evaluate h-1(mi) to relate it to hi, if no lag then JMLagTimeObs=JMTimeObs;
  JMLagTimeObs=max(JMTimeObs-&SharedLagging.,JMFromTimeObs);
  if &EventTimeVar. NE . then output;
  end;
  
*if not (ONLYLONGI in AdditionalOptions);%end;
run;

%end;
%else %if  &DataStructure. = VERTICAL %then %do;

**** Sorting data by subject ID;
proc sort data=_JMData;
  by JMDist &SubjectVar.  %if not (%elementinlist(ONLYEVENT, &AdditionalOptions.)) %then &LongiTimevar.;
                         %if not (%elementinlist(ONLYEVENT, &AdditionalOptions.)) %then &EventTimeVar.;
;
run;



 data _JMData;
  set _JMData;
  by JMDist &SubjectVar.;
  %if not (%elementinlist(ONLYEVENT, &AdditionalOptions.)) %then  %do; 
  if  JMDist = "LONGI" then do;
    JMTimeObs = 	&LongiTimevar.;
    JMResponse = 		&LongiVar.;
    if &LongiTimevar. NE . then output;
  end;
  %end;
  %if not (%elementinlist(ONLYLONGI, &AdditionalOptions.)) %then  %do; 
  if JMDist = "EVENT" then do;
  %if &EventTimeStart. NE %then %do; JMFromTimeObs = &EventTimeStart.; %end;
  %else %do; JMFromTimeObs = 0; %end;
  JMResponse = 		&EventTimeVar.;
  JMTimeObs = 	    &EventTimeVar.;
  JMEvent= 			&EventVar. = "&EventVal.";
  *JMLagTimeObs are the times points at which we will evaluate h-1(mi) to relate it to hi, if no lag then JMLagTimeObs=JMTimeObs;
  JMLagTimeObs=max(JMTimeObs-&SharedLagging.,JMFromTimeObs);
  if last.&SubjectVar. and &EventTimeVar. NE . then JMLastTimeEntry=1;
  if &EventTimeVar. NE . then output;
  end;
  %end;
 run;
%end;


**** Creating the stratification variable;
data _JMData;
  set _JMData;
  %if &EventStrata. NE %then %do; 
  JMStrata = &EventStrata.; 
  %end;
  %else %do; 
  JMStrata = ""; 
  JMStrataN=1;
  %end;
run;

**** Sorting data by JMDist JMStrata SubjectId  JMFromTimeObs JMTimeObs;
proc sort data=_JMData;
  by JMDist  JMStrata &SubjectVar.  %if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then JMFromTimeObs; JMTimeObs;
run;

**** Creating the numeric stratification variable ;
%if &EventStrata. NE %then %do;
 data _JMData;
  set _JMData end=endoffile;
  by JMDist JMStrata;
  if JMDist = "EVENT" and first.JMStrata then JMStrataN+1;
  if endoffile then call symputx('JMStrataN',JMStrataN);
 run;
%end;
%else %do; %let JMStrataN=1; %end;

**** Sorting data by JMDist JMStrata SubjectId  JMFromTimeObs JMTimeObs;
proc sort data=_JMData;
  by &SubjectVar. JMDist  JMStrata   %if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then JMFromTimeObs; JMTimeObs;
run;

%if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then  %do;

%let LogF=;
%if %elementinlist(&EventModel., WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;
%Let LogF=Log;
data _JMData;
  set _JMData;
  if JMDist = "EVENT" then JMLogTimeObs=log(JMTimeObs);
run;
%end;

%if %elementinlist(&EventModel., PIECEWISE GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;


* Calculate automatically the knots of the EVENT model in case they are not provided by the user; 
%if (&DataStructure. = VERTICAL and (not (%elementinlist(COMPETING, &AdditionalOptions.)))) %then %do;
%calculateknotspartition(indata=_JMData,var=JM&LogF.TimeObs, outdata=_JMCalculated_Knots_EVENT_Model,Number=&EventNknots., 
                           knotname=&LogF.Eventknot, wh = JMDist="EVENT" and JMLastTimeEntry=1 %if %elementinlist(EVENTTIMES, &AdditionalOptions.) %then and JMEvent=1;);  
%end; 

%else %if (&DataStructure. = VERTICAL and (%elementinlist(COMPETING, &AdditionalOptions.))) %then %do;
%calculateknotspartition(indata=_JMData,var=JM&LogF.TimeObs, outdata=_JMCalculated_Knots_EVENT_Model,Number=&EventNknots., 
                           knotname=&LogF.Eventknot, wh = JMDist="EVENT" and JMStrataN=1 %if %elementinlist(EVENTTIMES, &AdditionalOptions.) %then and JMEvent=1;);  
%end; 
%else %do;
%calculateknotspartition(indata=_JMData,var=JM&LogF.TimeObs, outdata=_JMCalculated_Knots_EVENT_Model,Number=&EventNknots., 
                           knotname=&LogF.Eventknot, wh = JMDist="EVENT" %if %elementinlist(EVENTTIMES, &AdditionalOptions.) %then and JMEvent=1; );  
%end;


*Check if repeated Time To Event knots have been produced;
 %do i = 2 %to &EventNknots.;
      %let j = %eval(&i - 1);
	  *Line below was updated in Macro Version 1.02 due to a bug;
      %let repeatedEventknots = &&&LogF.Eventknot&i. = &&&LogF.Eventknot&j.;
   %if &repeatedEventknots. %then %do;
      %PUT %str(ER)ROR: IN THE EVENT MODEL, THE JM MACRO HAS REPEATED TWO (OR MORE) KNOTS. IN ORDER TO AVOID THIS THERE ARE SEVERAL OPTIONS: 1.- REDUCING THE NUMBER OF KNOTS, 2.- USING THE EVENTTIMES OPTION, 3.- FIXING THE KNOTS MANUALLY.;
      %ABORT;
   %end;
 %end;
  
proc sql noprint;
   select put(min(JMTimeObs), hex16.), put(max(JMTimeObs), hex16.), put(log(min(JMTimeObs)), hex16.), put(log(max(JMTimeObs)), hex16.) 
           into :MinObservedEvent, :MaxObservedEvent, :LogMinObservedEvent, :LogMaxObservedEvent
   from _JMData where JMDist="EVENT" and JMEvent=1;
quit; 

proc sql noprint;
   select put(max(JMTimeObs), hex16.) into :MaxObservedTime
   from _JMData where JMDist="EVENT";
quit; 

%if (%elementinlist(&EventModel., WSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE)) and &EventLowerknot. = %then %do;
	%let LogEventLowerknot = &LogMinObservedEvent.;
%end;
%if (%elementinlist(&EventModel.,  GSPLINE GNATURALCUBIC  GBSPLINE)) and &EventLowerknot. = %then %do;
    %let EventLowerknot = &MinObservedEvent.;
%end;

%if (%elementinlist(&EventModel., WSPLINE WNATURALCUBIC WBSPLINE RPNATURALCUBIC RPSPLINE RPBSPLINE)) and  &EventUpperknot. = %then %do;
	%let LogEventUpperknot = &LogMaxObservedEvent.;
%end;
%if (%elementinlist(&EventModel., GSPLINE GNATURALCUBIC  GBSPLINE)) and  &EventUpperknot. = %then %do; 
	%let EventUpperknot = &MaxObservedEvent.;
%end;

* Maximum Time Event is defined as the Knot N+1;
  %let Eventknot%eval(&EventNknots.+1)=  &MaxObservedTime.;


 
%if &EventModel. = PIECEWISE  %then %do;
/************************************************************************************************/
/* Creation of the _JMPieces datasets where time rows will be broken down in difference pieces; */
/************************************************************************************************/
data _JMPrepieces;
  set _JMData;
  %if  (&DataStructure. = VERTICAL) and (not (%elementinlist(COMPETING, &AdditionalOptions.))) %then %do;
     if JMLastTimeEntry=1;
	 *Statement below added in version 1.02 to fix a bug if piecewise is used with vertical;
	 JMFromTimeObs=0;
	 *Statement below added in version 1.02;
	 %PUT NOTE: The disjoint time-to-event model (used to estimate initial parameters) only use the last time-to-event record of each subject.; 

  %end;
  %if  (&DataStructure. = VERTICAL) and (%elementinlist(COMPETING, &AdditionalOptions.)) %then %do;
     if JMStrataN=1;
	 *Statement below added in version 1.02;
	 %PUT NOTE: The disjoint time-to-event model (used to estimate initial parameters) only use data from the first stratum.; 
  %end;
run;

data _JMpieces (drop = stopped j JMFromTimeObs);
  set _JMPrepieces;
  if JMDist="LONGI" then output;
  else if JMDist="EVENT" then do;
  stopped=0;
  if JMResponse <=  input("&Eventknot1.",  hex16.) then do;
     piece=1;
     resp_piece=JMResponse;
	 event_piece=JMEvent;
	 JMFromTimeObs2=JMFromTimeObs;
	 stopped=1;
	 output;
  end;
  else do;
  	event_piece=0;
      if JMFromTimeObs < input("&Eventknot1.", hex16.) then do;
        piece=1;
	    JMFromTimeObs2=JMFromTimeObs;
        resp_piece=input("&Eventknot1.", hex16.)- JMFromTimeObs2;
 	    output;
	   end;
     do j=1 to &EventNknots.;
	   piece=j+1;
	   JMFromTimeObs2 = max(input(symget('Eventknot'!!left(put(j,1.))),hex16.), JMFromTimeObs);
	   if JMFromTimeObs < input(symget('Eventknot'!!left(put(j+1,1.))),hex16.) then do;
	     if stopped=0 and JMEvent=1 and  event_piece=0 and JMResponse <= input(symget('Eventknot'!!left(put(j+1,1.))),hex16.) then do;
           resp_piece = JMResponse - JMFromTimeObs2; 
           event_piece=1;
	       stopped=1;
	       output;
	     end;
	     else if stopped=0 and JMEvent=0 and event_piece=0 and  JMResponse <= input(symget('Eventknot'!!left(put(j+1,1.))),hex16.) then do;
           resp_piece=JMResponse - JMFromTimeObs2;
  	       event_piece=0;
	       stopped=1;
	       output;
         end;
         else if JMResponse > input(symget('Eventknot'!!left(put(j+1,1.))),hex16.) then do;
           resp_piece=input(symget('Eventknot'!!left(put(j+1,1.))),hex16.) - JMFromTimeObs2;
	       event_piece=0;
	       output;
	     end;
       end;*if JMFromTimeObs < ...;  
	 end;  *do j=1 to &EventNknots.;
   end;    *else do;
   end;    *else if JMDist="EVENT" then do;;
run;
%end;
%end;

*if not (ONLYLONGI in AdditionalOptions);
%end; 


**** Adding the 15 nodes and the 15 weights of the Kronrod Rule for the JMTimeObs (event/obs times); 
**** These variables are used both in the longitudinal model and in the time to event model;
data _JMData;
  set _JMData;
  %kronrodrule15p(from=JMFromTimeObs, to=JMTimeObs, nodes=JMTimeObs_kr_, weights=JM_kr_weight, tempor=1); 
run;


%if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then  %do;

%if %elementinlist(&EventModel., WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;
data _JMData;
  set _JMData;
  array JMTimeObs_kr_[15] ;
  array kr_JMLogTimeObs_[15] ;
  do r = 1 to 15;
   kr_JMLogTimeObs_[r] = log(JMTimeObs_kr_[r]);
  end;
  drop r;
run;
%end;


%if  (%elementinlist(&EventModel., WBSPLINE RPBSPLINE)) %then %do;
  proc sql noprint;
   select put(Log(min(JMTimeObs_kr_1)), hex16.) into :LogMin_kr_node
   from _JMData where JMDist="EVENT";
  quit;
%if (&EventModel. =  RPBSPLINE) and (&EventLowerknot. NE ) %then %do;
%if    (&LogEventLowerknot. < &LogMin_kr_node.)  %then %do;
   %PUT %str(ER)ROR: For the RPBSPLINE event model, the lower knot must be smaller or equal to %sysfunc(exp(input("&LogMin_kr_node.",hex16.))) ; 
   %ABORT;  
%end;
%end;
%if  &EventLowerknot. = %then %do;
  %let LogEventLowerknot = &LogMin_kr_node.;
%end;
%end;

%if %elementinlist(&EventModel., GBSPLINE WBSPLINE RPBSPLINE) %then %do;
  %bspline(indata=_JMData, outdata=_JMData, invariable=JM&LogF.TimeObs,  outarray=JMEvent_b_, 
          lowerknot=&&&LogF.Eventlowerknot., upperknot=&&&LogF.Eventupperknot., 
          posout=1, dimout=1, Nknots=&EventNknots., degree=&Eventdegree., knot1=&&&LogF.Eventknot1., 
          knot2=&&&LogF.Eventknot2., knot3=&&&LogF.Eventknot3., knot4=&&&LogF.Eventknot4., knot5=&&&LogF.Eventknot5., 
          knot6=&&&LogF.Eventknot6., knot7=&&&LogF.Eventknot7., knot8=&&&LogF.Eventknot8., knot9=&&&LogF.Eventknot9.,
		  incderiv=YES, incderiv2 = NO, if=JMDist='EVENT', nparam_var=event_nparam, dif=&SmallTimeDiff.);
   %do r = 1 %to 15;
    %bspline(indata=_JMData, outdata=_JMData, invariable= kr_JM&LogF.TimeObs_&r., outarray=kr_JMEvent_b_,  posout=&r.,
	 lowerknot=&&&LogF.Eventlowerknot., upperknot=&&&LogF.Eventupperknot., dimout=15, Nknots=&EventNknots., 
     knot1=&&&LogF.Eventknot1., knot2=&&&LogF.Eventknot2., knot3=&&&LogF.Eventknot3., knot4=&&&LogF.Eventknot4., knot5=&&&LogF.Eventknot5.,
     knot6=&&&LogF.Eventknot6., knot7=&&&LogF.Eventknot7., knot8=&&&LogF.Eventknot8., knot9=&&&LogF.Eventknot9., incderiv=YES,incderiv2 = NO, 
     degree=&Eventdegree., if=JMDist='EVENT', nparam_var=event_nparam, dif=&SmallTimeDiff.);
   %end;
%end;
%if %elementinlist(&EventModel., GNATURALCUBIC WNATURALCUBIC RPNATURALCUBIC) %then %do; 
    %ncspline(indata=_JMData, outdata=_JMData, invariable = JM&LogF.TimeObs,  outarray = JMEvent_b_, 
         posout=1, dimout=1, /*rmatrix=&rmatrix_first., rmatrixid = bevent,*/
         lowerknot=&&&LogF.Eventlowerknot., upperknot=&&&LogF.Eventupperknot., Nknots=&EventNknots., 
         knot1=&&&LogF.Eventknot1., knot2=&&&LogF.Eventknot2., knot3=&&&LogF.Eventknot3., knot4=&&&LogF.Eventknot4., knot5=&&&LogF.Eventknot5.,
         knot6=&&&LogF.Eventknot6., knot7=&&&LogF.Eventknot7., knot8=&&&LogF.Eventknot8., knot9=&&&LogF.Eventknot9.,
	 	 incderiv=YES, incderiv2 = NO, dif=&SmallTimeDiff.,nparam_var=event_nparam, if = JMDist='EVENT');
   %do r = 1 %to 15;
     %ncspline(indata=_JMData, outdata=_JMData, invariable= kr_JM&LogF.TimeObs_&r.,  outarray=kr_JMEvent_b_,  posout=&r.,
          dimout=15, 
          lowerknot=&&&LogF.Eventlowerknot., upperknot=&&&LogF.Eventupperknot., Nknots=&EventNknots.,
          knot1=&&&LogF.Eventknot1., knot2=&&&LogF.Eventknot2., knot3=&&&LogF.Eventknot3., knot4=&&&LogF.Eventknot4., knot5=&&&LogF.Eventknot5.,
          knot6=&&&LogF.Eventknot6., knot7=&&&LogF.Eventknot7., knot8=&&&LogF.Eventknot8., knot9=&&&LogF.Eventknot9.,
	     incderiv=YES, incderiv2 = NO, dif=&SmallTimeDiff., nparam_var=event_nparam, if = JMDist='EVENT');
   %end;
%end;
%if %elementinlist(&EventModel., GSPLINE WSPLINE RPSPLINE) %then %do; 
   %spline(indata=_JMData, outdata=_JMData, invariable = JM&LogF.TimeObs,   outarray = JMEvent_b_, posout=1, dimout=1, 
         lowerknot=&&&LogF.Eventlowerknot., upperknot=&&&LogF.Eventupperknot., Nknots=&EventNknots., degree=&EventDegree., 
         knot1=&&&LogF.Eventknot1., knot2=&&&LogF.Eventknot2., knot3=&&&LogF.Eventknot3., knot4=&&&LogF.Eventknot4., knot5=&&&LogF.Eventknot5.,
         knot6=&&&LogF.Eventknot6., knot7=&&&LogF.Eventknot7., knot8=&&&LogF.Eventknot8., knot9=&&&LogF.Eventknot9.,
	 	 incderiv=YES, incderiv2 = NO, dif=&SmallTimeDiff.,nparam_var=event_nparam, normalize=NO, if = JMDist='EVENT');
   %do r = 1 %to 15;
    %spline(indata=_JMData, outdata=_JMData, invariable= kr_JM&LogF.TimeObs_&r.,   outarray=kr_JMEvent_b_,  posout=&r.,
      dimout=15, lowerknot=&&&LogF.Eventlowerknot., upperknot=&&&LogF.Eventupperknot., Nknots=&EventNknots., degree=&EventDegree., 
         knot1=&&&LogF.Eventknot1., knot2=&&&LogF.Eventknot2., knot3=&&&LogF.Eventknot3., knot4=&&&LogF.Eventknot4., knot5=&&&LogF.Eventknot5.,
         knot6=&&&LogF.Eventknot6., knot7=&&&LogF.Eventknot7., knot8=&&&LogF.Eventknot8., knot9=&&&LogF.Eventknot9.,
	     incderiv=YES, incderiv2 = NO, dif=&SmallTimeDiff., nparam_var=event_nparam, normalize=NO, if = JMDist='EVENT');
   %end;
%end;


*if not (ONLYLONGI in AdditionalOptions);%end; 

%if not (%elementinlist(ONLYEVENT, &AdditionalOptions)) %then  %do;
*Obtain Knots for the Longitudinal Model if not given by user; 
%calculateknotspartition(indata=_JMData, var=JMTimeObs, outdata=_JMCalculated_Knots_Longi_Model, 
                Number=&LongiNknots., knotname=Longiknot, wh = JMDist="LONGI");
 
*Check if repeated Longitudinal knots have been produced;
%do i = 2 %to &LongiNknots.;
   %let j = %eval(&i - 1);
   %let repeatedlongiknots = &&Longiknot&i. = &&Longiknot&j.;
   %if &repeatedlongiknots. %then %do;
      %PUT %str(ER)ROR: IN THE LONGITUDINAL MODEL, THE JM MACRO HAS CREATED TWO KNOTS WITH THE SAME VALUE REDUCE THE NUMBER OF KNOTS OR FIX THE KNOTS MANUALLY;
      %ABORT;
   %end;
%end;

				
**** Macro Variable incderiv used to request derivatives to the bspline and spline macros;
%let incderiv=NO;
%if %elementinlist(SLOPE, &SharedParam.) %then %do;
  %let incderiv=YES;
%end;
********** Calculate Minimum and Maximum OBserved Time in the Longitudinal Model;
********      NOTE: Times when JMDist="EVENT" are NOT taken into account  ***********;
 proc sql noprint;
   select put(min(JMTimeObs),hex16.), put(max(JMTimeObs),hex16.) into :LONGITimeMin, :LONGITimeMax
   from _JMData where JMDist = "LONGI"; 
 quit;
  
********** If not given by the user;
********** Calculate Lower and Upper Bondaries to be used in the Longitidunal Model;
********** They are used by the BSPLINE, SPLINE AND NCSPLINE MACROS****************;;
%if &Longilowerknot. = %then %do; %let Longilowerknot = %sysfunc(putn(0 ,hex16.)); %end;
%if &Longiupperknot. = %then %do; %let Longiupperknot = &LONGITimeMax.; %end;
*Calculate bspline basis functions for the Longitudinal Model ; 
%if &LongiTimeModel. = BSPLINE %then %do; 
  %bspline(indata=_JMData, outdata=_JMData, invariable = JMTimeObs,  outarray = JMTimeObs_b_, 
         posout=1, dimout=1, 
         lowerknot=&Longilowerknot., upperknot=&Longiupperknot., Nknots=&LongiNknots., degree=&Longidegree., 
         knot1=&Longiknot1., knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4.,knot5=&Longiknot5.,
         knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
	 	 incderiv=NO, incderiv2 = NO,dif=&SmallTimeDiff., if = JMDist='LONGI');
  %if not (%elementinlist(ONLYLONGI, &AdditionalOptions.)) %then  %do;
  %bspline(indata=_JMData, outdata=_JMData, invariable = JMLagTimeObs,  outarray = JMLagTimeObs_b_, 
         posout=1, dimout=1, 
         lowerknot=&Longilowerknot., upperknot=&Longiupperknot., Nknots=&LongiNknots., degree=&Longidegree., 
         knot1=&Longiknot1., knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4.,knot5=&Longiknot5.,
         knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
	 	 incderiv=&incderiv., incderiv2 = NO,dif=&SmallTimeDiff.);
   %if %elementinlist(CUMULATIVE, &SharedParam.) %then %do;
   %bspline(indata=_JMData, outdata=_JMData, invariable = JMFromTimeObs,  outarray = JMFromTimeObs_b_, 
         posout=1, dimout=1, 
         lowerknot=&Longilowerknot., upperknot=&Longiupperknot., Nknots=&LongiNknots., degree=&Longidegree., 
         knot1=&Longiknot1., knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4.,knot5=&Longiknot5.,
         knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
	 	 incderiv=NO, incderiv2 = NO,dif=&SmallTimeDiff.);
   %end;
   %end;
%end;
%else %if &LongiTimeModel. = SPLINE %then %do; 
  %spline(indata=_JMData, outdata=_JMData, invariable=JMTimeObs,  outarray=JMTimeObs_b_, 
          lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
          posout=1, dimout=1, Nknots=&LongiNknots., degree=&Longidegree., knot1=&Longiknot1., 
          knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4., knot5=&Longiknot5., 
          knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
		  incderiv=NO, incderiv2 = NO, dif=&SmallTimeDiff., normalize=NO, if = JMDist='LONGI'); 
  %if not (%elementinlist(ONLYLONGI, &AdditionalOptions.)) %then  %do;
  %spline(indata=_JMData, outdata=_JMData, invariable=JMLagTimeObs,  outarray=JMLagTimeObs_b_, 
          lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
          posout=1, dimout=1, Nknots=&LongiNknots., degree=&Longidegree., knot1=&Longiknot1., 
          knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4., knot5=&Longiknot5., 
          knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
		  incderiv=&incderiv., incderiv2 = NO, dif=&SmallTimeDiff., normalize=NO); 
   %if %elementinlist(CUMULATIVE, &SharedParam.) %then %do;
   %spline(indata=_JMData, outdata=_JMData, invariable=JMFromTimeObs,  outarray=JMFromTimeObs_b_, 
          lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
          posout=1, dimout=1, Nknots=&LongiNknots., degree=&Longidegree., knot1=&Longiknot1., 
          knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4., knot5=&Longiknot5., 
          knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
		  incderiv=NO, incderiv2 = NO, dif=&SmallTimeDiff., normalize=NO); 
   %end;
   %end;
%end; 
%else %if &LongiTimeModel. =  NATURALCUBIC %then %do; 

  %ncspline(indata=_JMData, outdata=_JMData, invariable=JMTimeObs,  outarray=JMTimeObs_b_, 
          lowerknot=&Longilowerknot., upperknot=&Longiupperknot.,
          posout=1, dimout=1, Nknots=&LongiNknots., knot1=&Longiknot1., 
          knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4., knot5=&Longiknot5., 
          knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
		  incderiv=NO, incderiv2 = NO, dif=&SmallTimeDiff., if = JMDist='LONGI'); 
  %if not (%elementinlist(ONLYLONGI, &AdditionalOptions.)) %then  %do;
  %ncspline(indata=_JMData, outdata=_JMData, invariable=JMLagTimeObs,  outarray=JMLagTimeObs_b_, 
          lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
          posout=1, dimout=1, Nknots=&LongiNknots., knot1=&Longiknot1., 
          knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4., knot5=&Longiknot5., 
          knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
		  incderiv=&incderiv., incderiv2 = NO, dif=&SmallTimeDiff.); 
   %if %elementinlist(CUMULATIVE, &SharedParam.) %then %do;
   %ncspline(indata=_JMData, outdata=_JMData, invariable=JMFromTimeObs,  outarray=JMFromTimeObs_b_, 
          lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
          posout=1, dimout=1, Nknots=&LongiNknots., knot1=&Longiknot1., 
          knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4., knot5=&Longiknot5., 
          knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
		  incderiv=NO, incderiv2 = NO, dif=&SmallTimeDiff.); 
   %end;
   %end;
%end; 
%else %if &LongiTimeModel. = LINEAR %then %do; 
  %spline(indata=_JMData, outdata=_JMData, invariable=JMTimeObs,  outarray=JMTimeObs_b_, 
          lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
          posout=1, dimout=1, degree=1, Nknots=0, incderiv=NO, incderiv2 = NO, dif=&SmallTimeDiff., normalize=NO); 
  %if not (%elementinlist(ONLYLONGI, &AdditionalOptions.)) %then  %do;
  %spline(indata=_JMData, outdata=_JMData, invariable=JMLagTimeObs,  outarray=JMLagTimeObs_b_, 
          lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
          posout=1, dimout=1, degree=1, Nknots=0, incderiv=&incderiv., incderiv2 = NO, dif=&SmallTimeDiff., normalize=NO); 
   %if %elementinlist(CUMULATIVE, &SharedParam.) %then %do;
        %spline(indata=_JMData, outdata=_JMData, invariable=JMFromTimeObs,  outarray=JMFromTimeObs_b_, 
          lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
          posout=1, dimout=1, degree=1, Nknots=0, incderiv=NO, incderiv2 = NO, dif=&SmallTimeDiff., normalize=NO); 
   %end;
   %end;
%end;



**** Adding new variablew with lagging to each of the 15 nodes of the Kronrod Rule creeated before; 
data _JMData;
  set _JMData;
  array JMLagTimeObs_kr_[15] ;
  array JMTimeObs_kr_[15] ;
  do r = 1 to 15;
   JMLagTimeObs_kr_[r] = max(JMTimeObs_kr_[r]-&SharedLagging.,JMFromTimeObs);
  end;
  drop r;
run;

%if &KronrodBasis.  = YES %then %do; 
**** Adding the Bspline Basis to each of the nodes of the Kronrod Rule;
**** This step adds &longi_nparam. x 15 variables to the dataset!; 

%if &LongiTimeModel. = BSPLINE %then %do;
   %do r = 1 %to 15;
     %bspline(indata=_JMData, outdata=_JMData, invariable= JMLagTimeObs_kr_&r., outarray=JMTimeObs_kr_b_,  posout=&r.,
      dimout=15, lowerknot=&LongiLowerknot., upperknot=&LongiUpperknot., Nknots=&LongiNknots., degree=&LongiDegree., 
      knot1=&Longiknot1., knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4.,knot5=&Longiknot5.,
      knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9.,
	  incderiv=&incderiv., incderiv2 = NO);
   %end;
%end;
%else %if &LongiTimeModel. = SPLINE %then %do;
   %do r = 1 %to 15;
    %spline(indata=_JMData, outdata=_JMData, invariable= JMLagTimeObs_kr_&r., outarray=JMTimeObs_kr_b_,  posout=&r.,
	 lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
     dimout=15, Nknots=&LongiNknots., knot1=&Longiknot1., knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4.,knot5=&Longiknot5.,
      knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9., incderiv=&incderiv.,incderiv2 = NO, 
      degree=&Longidegree., normalize=NO);
   %end;
%end;
%else %if &LongiTimeModel. = NATURALCUBIC %then %do;
   %do r = 1 %to 15;
    %ncspline(indata=_JMData, outdata=_JMData, invariable= JMLagTimeObs_kr_&r., outarray=JMTimeObs_kr_b_,  posout=&r.,
	 lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
	 dimout=15, Nknots=&LongiNknots., knot1=&Longiknot1., knot2=&Longiknot2., knot3=&Longiknot3., knot4=&Longiknot4.,knot5=&Longiknot5.,
      knot6=&Longiknot6., knot7=&Longiknot7., knot8=&Longiknot8., knot9=&Longiknot9., incderiv=&incderiv.,incderiv2 = NO);
   %end;
%end;
%else %if &LongiTimeModel. = LINEAR %then %do;
   %do r = 1 %to 15;
    %spline(indata=_JMData, outdata=_JMData, invariable= JMLagTimeObs_kr_&r., outarray=JMTimeObs_kr_b_, 
	 lowerknot=&Longilowerknot., upperknot=&Longiupperknot., 
     posout=&r., dimout=15, degree=1, Nknots=0, incderiv=&incderiv., incderiv2 = NO, normalize=NO);
   %end;
%end;

%end;
%if %elementinlist(CUMULATIVE, &SharedParam.) %then %do;
 data _JMData;
   set _JMData;
       array JMLagTimeObs_kr_ [15] JMLagTimeObs_kr_1-JMLagTimeObs_kr_15; 
	   array JMFromTimeObs_b_[1,%eval(&longi_nparam.)] JMFromTimeObs_b_0-JMFromTimeObs_b_%eval(&longi_nparam.-1); 
       array JM_kr_weight[15] JM_kr_weight1-JM_kr_weight15;
	   array cm_Bin[%eval(&longi_nparam.-1)] cm_Bin1-cm_Bin%eval(&longi_nparam.-1);
	   array JMTimeObs_kr_b_[15,&longi_nparam.] JMTimeObs_kr_b_0-JMTimeObs_kr_b_%eval(&longi_nparam.*15-1);  
		do i = 1 to %eval(&longi_nparam.-1);
		  cm_Bin[i]=0;
          do r = 1 to 15;
           cm_Bin[i] = cm_Bin[i] + JMTimeObs_kr_b_[r,i+1]*JM_kr_weight[r]; 
          end;
		   cm_Bin[i] = cm_Bin[i]*(JMTimeObs-JMFromTimeObs)*0.5;
		end;
       array cm_JMTimeObs_b_[15,%eval(&longi_nparam.-1)] cm_JMTimeObs_b_1-cm_JMTimeObs_b_%eval((&longi_nparam.-1)*15);  
		do i = 1 to %eval(&longi_nparam.-1);
          do r = 1 to 15;
			 cm_JMTimeObs_b_[r, i] =  0.5*(JMFromTimeObs_b_[1,i+1]+JMTimeObs_kr_b_[1,i+1])*(JMLagTimeObs_kr_[1]-JMFromTimeObs); 
			 do s = 2 to r;
               cm_JMTimeObs_b_[r, i] = cm_JMTimeObs_b_[r, i] + 0.5*(JMTimeObs_kr_b_[s-1,i+1]+JMTimeObs_kr_b_[s,i+1])*(JMLagTimeObs_kr_[s]-JMLagTimeObs_kr_ [s-1]); 
             end;
          end;
		end;
       drop i r s;
   run;
%end; 


***** In this section the terms that interact with the time parameters are created to be placed into NLMIXED;

%let LonTimIntTerms = ;
%if &LongiTimeInter. NE %then %do;
  %let Number_LongiTimeInters = 0;
  %do %while(%qscan(&LongiTimeInter.,&Number_LongiTimeInters+1,%str( )) ne %str());
    %let Number_LongiTimeInters = %eval(&Number_LongiTimeInters+1);
  %end;
  %put The user has specified &Number_LongiTimeInters variables that interact with time;
  %do i = 1 %to &Number_LongiTimeInters;
    %let LongiTimeInter_&i =  %scan(&LongiTimeInter. ,&i) ;
	   %if   %length (&&LongiTimeInter_&i.) > 6 %then %do;
         %PUT %str(ER)ROR: The name of variable assigned to the LongiTimeInter parameter must have a maximum length of 6 characters; 
         %ABORT;  
       %end;
	****** Check that these variables exist*******; 
    %VarExist(&data.,&&LongiTimeInter_&i.);
    %do k = 1 %to %eval(&longi_nparam.-1);
       %let LonTimIntTerms = &LonTimIntTerms. JMTimeObs_b_&k*&&LongiTimeInter_&i.; 
    %end;
    %put Interaction Variable Number &i. is  &&LongiTimeInter_&i.;
  %end;
%put The following effects will be added to the Mixed Proc: &LonTimIntTerms.; 
%end;


* Macro Variables to control the situation with no intercept in the longitudinal model;

%if %elementinlist(NOINT, &AdditionalOptions.) %then  %do;
  %let noint=noint; 
  %let int=; 
  %let firsti=1;
  %let one_m_firsti=0; 
  %let inttxt =(without intercept);
%end;
%else %do;
  %let noint=;
  %let int=int; 
  %let firsti=0;
  %let one_m_firsti=1; 
  %let inttxt =;
%end;

*if not (%elementinlist(ONLYEVENT, &AdditionalOptions));%end;




%if &InitialParameters. NE DISJOINT %then %goto skipdisjointmodeling;

**** Closing the listing ouput, it will be opened again later to output only most relevant info; 
%if NOT (%elementinlist(UNTOUCHEDLISTING, &AdditionalOptions.)) %then %do;
  ods listing close;
%end;






%if not (%elementinlist(ONLYEVENT, &AdditionalOptions)) %then  %do;
************************************************************;
************ Fitting  the longitudinal Model ***************; 
************************************************************;
%let gc= ;
%if &LongiGMatrixParam. = CHOL %then %do;
  ods output cholg=_JMCholg;
  %let gc= gc;
%end;

%if &LongiType. = NORMAL  %then %do;

ods output SolutionF=_JMSolutionF ;
ods output covparms=_JMcovparms;
proc mixed data=_JMData &LongiModelOptions.; 
      class &SubjectVar.;
      model &LongiVar. = JMTimeObs_b_1-JMTimeObs_b_%eval(&longi_nparam.-1)   &LonTimIntTerms. &LongiCovariates. /cl &noint. ;
      random   &int. JMTimeObs_b_1-JMTimeObs_b_%eval(&longi_nparam.-1) /sub=&SubjectVar. type=&LongiGMatrix. &gc.; 
     where JMDist="LONGI";;
run;
%if %sysfunc(exist(_JMSolutionF))=0 %then %do;
   %PUT %str(ER)ROR: PROC MIXED DID NOT CONVERGE - THE REST OF THE JM MACRO HAS BEEN ABORTED;%abort;
%end;
%end;
%else %if %elementinlist(&LongiType., BINARY BINARY2) %then %do;

ods output ParameterEstimates=_JMSolutionF ;
ods output covparms=_JMcovparms;
proc glimmix data=_JMData  &LongiModelOptions.; 
      nloptions &LongiGLINLOptions.;
      class &SubjectVar.;
      model &LongiVar. (Event="&LongiEventCat.") =  JMTimeObs_b_1-JMTimeObs_b_%eval(&longi_nparam.-1)  &LonTimIntTerms. &LongiCovariates. /  &noint. distribution= binary  link=&LongiLink. solution;
      random    &int. JMTimeObs_b_1-JMTimeObs_b_%eval(&longi_nparam.-1)/ sub=&SubjectVar. type=&LongiGMatrix. &gc.; 
     where JMDist="LONGI";;
run;
%if %sysfunc(exist(_JMSolutionF))=0 %then %do;
   %PUT %str(ER)ROR: PROC GLIMMIX DID NOT CONVERGE - THE REST OF THE JM MACRO HAS BEEN ABORTED;%abort;
%end;
%end;
%else %if &LongiType. = BINOMIAL  %then %do;

ods output ParameterEstimates=_JMSolutionF ;
ods output covparms=_JMcovparms;
proc glimmix data=_JMData  &LongiGLIOptions.; 
      nloptions &LongiGLINLOptions.;
      class &SubjectVar.;
      model &LongiVar./&LongiTrialsVar. =  JMTimeObs_b_1-JMTimeObs_b_%eval(&longi_nparam.-1)  &LonTimIntTerms. &LongiCovariates. / &noint. distribution= binomial  link=&LongiLink. solution;
      random    &int. JMTimeObs_b_1-JMTimeObs_b_%eval(&longi_nparam.-1)/ sub=&SubjectVar. type=&LongiGMatrix. &gc.; 
     where JMDist="LONGI";;
run;
%if %sysfunc(exist(_JMSolutionF))=0 %then %do;
   %PUT %str(ER)ROR: PROC GLIMMIX DID NOT CONVERGE - THE REST OF THE JM MACRO HAS BEEN ABORTED;%abort;
%end;
%end;

%if &LongiGMatrixParam. = CHOL and %sysfunc(exist(_JMCholg))=0 %then %do;
   %PUT %str(ER)ROR: PROC MIXED/GLIMMIX DID NOT CREATE THE CHOLESKI DECOMPOSITION OF THE G MATRIX - THE REST OF THE JM MACRO HAS BEEN ABORTED;%abort;
%end;

**** Arrange the point estimate parameters of the longitudinal Model; 
data _JMsolutionf2 (keep = parameter estimate );
  length effect $100;
  set _JMsolutionf;
  effect=upcase(effect);
  effect=TRANWRD(effect, "JMTIMEOBS_B_","TIME_B");;
  effect=TRANWRD(effect, "*","_BY_");;
  effect=compress("L_"||effect);
  rename effect=parameter;
run;

**** Arrange the variance  parameters of the longitudinal Model; 
data _JMcovparms2 (keep = parameter estimate);
  length covparm $100;
  set _JMcovparms;
  covparm=upcase(covparm);
  %if "&LongiGMatrix." = "VC" %then %do;
  covparm=TRANWRD(covparm, "JMTIMEOBS_B_", "TIME_B");
  sd=1;
  %end;
  %else %if "&LongiGMatrix." = "UN" %then %do;
  if substr(covparm,4,1)=substr(covparm,6,1)=1 and "&noint." ne "noint" then do;
      covparm="INTERCEPT";
      sd=1;
  end;
  else if substr(covparm,4,1)=substr(covparm,6,1) then do;
      covparm=compress("TIME_B"||put((input(substr(covparm,4,1),1.)-&one_m_firsti.),1.));
	  sd=1;
  end;
  else if substr(covparm,1,3)= "UN(" then do;
      covparm=compress("L_COV_B"||put((input(substr(covparm,6,1),1.)-&one_m_firsti.),1.)||"_B"||put((input(substr(covparm,4,1),1.)-&one_m_firsti.),1.));
	  sd=0;
  end;
  else if covparm= "RESIDUAL" then sd=1;
  else if index("&AdditionalOptions.","NOMACROWARN") = 0 then put "WAR" "NING: ONE OF THE COMPONENTS OF THE COVARIANCE PARAMETERS DATASET WAS NOT EXPECTED";
  %end;
  %else %if "&LongiGMatrix." = "ANTE(1)" %then %do;
  if covparm="VAR(1)" and "&noint." ne "noint" then do;
     covparm="INTERCEPT";
     sd=1;
  end;
  else if substr(covparm,1,4)= "VAR(" then do;
      covparm=compress("TIME_B"||put((input(substr(covparm,5,1),1.)-&one_m_firsti.),1.));
	  sd=1;
  end;
  else if substr(covparm,1,4)= "RHO(" then do;
      covparm=compress("L_RHO_B"||put((input(substr(covparm,5,1),1.)-&one_m_firsti.),1.)||"_B"||put((input(substr(covparm,5,1),1.)-&one_m_firsti.+1),1.));
	  sd=0;
  end;
  else if covparm= "RESIDUAL" then sd=1;
  else if index("&AdditionalOptions.","NOMACROWARN") = 0 then put "WAR" "NING: ONE OF THE COMPONENTS OF THE COVARIANCE PARAMETERS DATASET WAS NOT EXPECTED";
  %end;
  if sd=1 then do;
     covparm=compress("L_LOGSD_"!!covparm);
     if sqrt(estimate) > 1e-8 then estimate = log(sqrt(estimate));
     else estimate=&LongiSmallLogSD.;
  end;
  rename covparm=parameter;
run;

%if &LongiGMatrixParam. = CHOL %then %do;
data   _JMCholg2 (keep = parameter estimate);
 set  _JMCholg;
 array cols[&longi_nparam.] col1-col&longi_nparam.;
 do j =1 to Row;
   parameter=compress("L_GCHOL_B_"||put(row,1.)||"_"||put(j,1.));
   estimate=cols[j];
   if row = j or "&LongiGMatrix." = "UN" then output; 
 end;
run;
 data _JMcovparms2 (keep = parameter estimate);
   set _JMcovparms2 _JMCholg2;
   if parameter = "L_LOGSD_RESIDUAL" or index(parameter,"L_GCHOL_B_") > 0;
 run;

%end;

*if not (ONLYEVENT in AdditionalOptions);%end; 


%if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then  %do;
**************************************************************************************;
************ Fitting the Time to Event Model for the initial parameters **************; 
**************************************************************************************;
%let InParEventModel = &EventModel.;
%if %elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;
  %let InParEventModel = EXPONENTIAL;
%end;
*Statements below modified in version 1.04;
%if  %elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;
   %PUT %str(NO)TE: A simplified time-to-event model has been fitted that assumes constant hazard. NLMIXED will use these estimates as initial parameters for the joint model.; 
%end;
*Statements below modified in version 1.04;
%if   "&EventStrata." NE "" %then %do;
   %PUT %str(NO)TE: A simplified time-to-event model has been fitted that does not include the stratification factor. NLMIXED will use these estimates as initial parameters for the joint model.; 
%end;
%if &InParEventModel.= WEIBULL or &InParEventModel.= EXPONENTIAL %then %do;  
ods output ParameterEstimates=_JMParameterEstimates;
 proc lifereg data=_JMData;
 model JMResponse*JMEvent(0) = &EventCovariates. /dist= &InParEventModel.;  
 where JMDist="EVENT" %if  &DataStructure. = VERTICAL and not %elementinlist(COMPETING, &AdditionalOptions.) %then and JMLastTimeEntry=1;
                      %if  &DataStructure. = VERTICAL and %elementinlist(COMPETING, &AdditionalOptions.) %then and JMStrataN=1;
; 
run; 
%end;

%else %if &InParEventModel.= PIECEWISE %then %do;

data _JMpieces2;
  set _JMpieces;
                     *Statement below modified in version 1.04;
  if JMDist="EVENT"   %if  &DataStructure. = VERTICAL and not %elementinlist(COMPETING, &AdditionalOptions.) %then and JMLastTimeEntry=1;
                      %if  &DataStructure. = VERTICAL and %elementinlist(COMPETING, &AdditionalOptions.) %then and JMStrataN=1;
  ;
run;
 
proc sort data=_JMpieces2;
  by piece;
run;

ods output ParameterEstimates=_JMParameterEstimates;
proc lifereg data=_JMpieces2;
   class piece;
   model resp_piece*event_piece(0) = piece &EventCovariates. /dist=exponential;  
run; 
%end;


***** Creation of a variable with the EVENT Model Covariates quoted and with commas;
%if &EventCovariates. NE  %then %do;
  %let Number_EventCovariates=0;
  %let EventCovariatesQuotedCommas = ;
  %do %while(%qscan(&EventCovariates.,&Number_EventCovariates+1,%str( )) ne %str());
    %let Number_EventCovariates = %eval(&Number_EventCovariates+1);
  %end;
  %put The user has specified &Number_EventCovariates covariates for the time to event model;
  %do i = 1 %to &Number_EventCovariates;
    %let EVENTCovariate_&i. =  %scan(&EventCovariates. ,&i) ;
	****** Check that these variables exist *******; 
    %VarExist(&data.,&&EVENTCovariate_&i.);
	%put EVENTCovariate_ number &i is &&EVENTCovariate_&i.;
	%if &i  >1 %then %let EventCovariatesQuotedCommas = &EventCovariatesQuotedCommas,;; 
    %let EventCovariatesQuotedCommas =  &EventCovariatesQuotedCommas.   "&&EVENTCovariate_&i.";
  %end;
%end;

%if &EventModel. = WEIBULL  %then %do;
proc sql noprint; 
select estimate into :scale from _JMParameterEstimates where parameter="Weibull Shape";
  quit;
%end;
%else %let scale=1;

%if (%elementinlist(&EventModel., EXPONENTIAL WEIBULL PIECEWISE)) or &EventCovariates. NE %str() %then %do;
**** Arrange the  parameters of the Time to Event Model; 
data _JMParameterEstimates2 (keep = parameter estimate CovariateF);
  length parameter $100; 
  %if &EventModel. = PIECEWISE   %then  %do; retain int; %end;
  set _JMParameterEstimates
      (where=(upcase(parameter) in (                      %if &EventModel. = EXPONENTIAL  %then "INTERCEPT" ,; 
                                                          %if &EventModel. = WEIBULL      %then "INTERCEPT" , "WEIBULL SHAPE" ,;  
                                                          %if &EventModel. = PIECEWISE    %then "INTERCEPT", "PIECE" ,;
                                                          %if &EventCovariates. NE %str() %then  &EventCovariatesQuotedCommas.;
												          
                                                  )));
  parameter=upcase(parameter);
  CovariateF=0;
  %if &EventCovariates. NE %str() %then %do; 
     if upcase(parameter) in (&EventCovariatesQuotedCommas.) then do;
       parameter=compress("E_"!!parameter); 
       estimate=-estimate*&scale.;
	   CovariateF=1;
	 end;
  %end;

  %if &InParEventModel. = WEIBULL  %then %do;
  if parameter ="WEIBULL SHAPE" then do;
	  *WEIBULL SHAPE is log-transformed;
      parameter=TRANWRD(parameter, "WEIBULL", "");
      parameter=compress("E_LOG"!!parameter);
      if estimate > 1e-8 then estimate=log(estimate);
      else estimate = -1e20;
  end;
  parameter=TRANWRD(parameter, "INTERCEPT", "E_LOGSCALE");
  output;
  %end;

  %else %if &InParEventModel. = EXPONENTIAL  %then %do;
  if parameter="INTERCEPT" then do;
     parameter="E_LOGHAZARD";
     estimate=-estimate;
  end;
  output;
  %end;
  %else %if &InParEventModel. = PIECEWISE %then %do;
  if parameter="INTERCEPT" then int=estimate;
    else  do;
    if parameter="PIECE" then do; 
       estimate=estimate+int; 
       parameter=TRANWRD(parameter, "PIECE", "E_LOGHAZARD"); 
       parameter=compress(parameter!!"_"!!Level1); 
	   estimate=-estimate;
	end;
   output;
   end;
  %end;
run;
%end;
%if (%elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE)) %then %do;
proc sql noprint;
select -estimate into :logrisk
    from _JMParameterEstimates 
     where upcase(parameter)="INTERCEPT";
quit;

data _JMDummyEventData;
  do &LogF.Time = input("&&&LogF.Eventlowerknot.",hex16.) to input("&&&LogF.Eventupperknot.",hex16.) by &SmallTimeDiff.;
  output;
end;
run;

%if %elementinlist(&EventModel., GBSPLINE WBSPLINE RPBSPLINE) %then %do;
     %bspline(indata=_JMDummyEventData, outdata=_JMDummyEventData, invariable=&LogF.Time,  outarray=JMEvent_b_, 
          lowerknot=&&&LogF.Eventlowerknot., upperknot=&&&LogF.Eventupperknot., 
          posout=1, dimout=1, Nknots=&EventNknots., degree=&Eventdegree., knot1=&&&LogF.Eventknot1., 
          knot2=&&&LogF.Eventknot2., knot3=&&&LogF.Eventknot3., knot4=&&&LogF.Eventknot4., knot5=&&&LogF.Eventknot5., 
          knot6=&&&LogF.Eventknot6., knot7=&&&LogF.Eventknot7., knot8=&&&LogF.Eventknot8., knot9=&&&LogF.Eventknot9.,
		  incderiv=NO, incderiv2 = NO, nparam_var=event_nparam, dif=&SmallTimeDiff.);
%end;
%if %elementinlist(&EventModel., GNATURALCUBIC WNATURALCUBIC RPNATURALCUBIC) %then %do; 
    %ncspline(indata=_JMDummyEventData, outdata=_JMDummyEventData, invariable = &LogF.Time,  outarray = JMEvent_b_, 
         posout=1, dimout=1, 
         lowerknot=&&&LogF.Eventlowerknot., upperknot=&&&LogF.Eventupperknot., Nknots=&EventNknots., 
         knot1=&&&LogF.Eventknot1., knot2=&&&LogF.Eventknot2., knot3=&&&LogF.Eventknot3., knot4=&&&LogF.Eventknot4., knot5=&&&LogF.Eventknot5.,
         knot6=&&&LogF.Eventknot6., knot7=&&&LogF.Eventknot7., knot8=&&&LogF.Eventknot8., knot9=&&&LogF.Eventknot9.,
	 	 incderiv=NO, incderiv2 = NO, dif=&SmallTimeDiff.,nparam_var=event_nparam);
%end;
%if %elementinlist(&EventModel., GSPLINE WSPLINE RPSPLINE) %then %do; 
   %spline(indata=_JMDummyEventData, outdata=_JMDummyEventData, invariable = &LogF.Time,   outarray = JMEvent_b_, posout=1, dimout=1, 
         lowerknot=&&&LogF.Eventlowerknot., upperknot=&&&LogF.Eventupperknot., Nknots=&EventNknots., degree=&EventDegree., 
         knot1=&&&LogF.Eventknot1., knot2=&&&LogF.Eventknot2., knot3=&&&LogF.Eventknot3., knot4=&&&LogF.Eventknot4., knot5=&&&LogF.Eventknot5.,
         knot6=&&&LogF.Eventknot6., knot7=&&&LogF.Eventknot7., knot8=&&&LogF.Eventknot8., knot9=&&&LogF.Eventknot9.,
	 	 incderiv=NO, incderiv2 = NO, dif=&SmallTimeDiff.,nparam_var=event_nparam, normalize=NO);
%end;

data _JMDummyEventData;
  set _JMDummyEventData;
  %if (%elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC)) %then %do;
    Dependent=&logrisk.;
  %end;
  %if (%elementinlist(&EventModel., RPNATURALCUBIC RPSPLINE RPBSPLINE)) %then %do;
    Dependent=&logrisk. + &LogF.time;
  %end;
run;

proc reg data=_JMDummyEventData outest=_JMSplinesParam noprint; 
  model Dependent =  JMEvent_b_0-JMEvent_b_%eval(&event_nparam.-1) /NOINT ;
run;


data _JMSplinesParam2 (keep = parameter estimate);
set _JMSplinesParam;
length parameter $100;
  %do i = 0 %to   %eval(&event_nparam.-1);  
    parameter="E_TIME_B&i.";
    estimate=JMEvent_b_&i.;
    output;
  %end;
%end;


%if (( %elementinlist(&EventModel., EXPONENTIAL WEIBULL PIECEWISE)) or (&EventCovariates. NE %str())) and (&JMStrataN. > 1) %then %do;
data _JMParameterEstimates2 ;
  set  _JMParameterEstimates2 ;
  if CovariateF=0 then do;
    temp=parameter;
    do s=1 to  &JMStrataN.;
	  parameter=compress(temp!!"_S"!!put(s,3.));
	  output;
    end;
  end;
  else if CovariateF=1 then output;
  drop temp;
run;
%end;
%if (%elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE)) and (&JMStrataN. > 1) %then %do;
data _JMSplinesParam2;
  set  _JMSplinesParam2;
    temp=parameter;
    do s=1 to  &JMStrataN.;
	  parameter=compress(temp!!"_S"!!put(s,3.));
	  output;
    end;
drop temp;
run;
%end;

*if not (ONLYLONGI in AdditionalOptions);%end; 





%if &SharedParam. NE NONE %then %do; 

%let Number_SharedParamInt=0; 
%if &SharedParamInt. NE %then %do;
  %do %while(%qscan(&SharedParamInt.,&Number_SharedParamInt+1,%str( )) ne %str());
    %let Number_SharedParamInt = %eval(&Number_SharedParamInt+1);
  %end;
  %do i = 1 %to &Number_SharedParamInt. ;
  	%let SharedParamInt_&i. =  %scan(&SharedParamInt. ,&i.) ;
	******* Check that these variables exist *******; 
    %VarExist(&data.,&&SharedParamInt_&i.);
  %end;
%end;
**** Initialitate the Shared-Parameter to zero; 
data _JMassoct;  
length parameter $100;  
%if %elementinlist(CURRENT_VALUE, &SharedParam.) %then %do;
  parameter="ASSOCT_TD";
  estimate=0;
  output;
  %do i=1 %to &Number_SharedParamInt.;
  parameter="ASSOCT_TD_&i";
  estimate=0;
  output;
  %end;
 %end;
%if %elementinlist(BASELINE_VALUE, &SharedParam.) %then %do;
  parameter="ASSOCT_BS";
  estimate=0;
  output;
  %do i=1 %to &Number_SharedParamInt.;
  parameter="ASSOCT_BS_&i";
  estimate=0;
  output;
  %end;
 %end;
%if %elementinlist(CHANGE_FROM_BASELINE, &SharedParam.) %then %do;
  parameter="ASSOCT_CH";
  estimate=0;
  output;
  %do i=1 %to &Number_SharedParamInt.;
  parameter="ASSOCT_CH_&i";
  estimate=0;
  output;
  %end;
 %end;
%if %elementinlist(SLOPE, &SharedParam.) %then %do;
  parameter="ASSOCT_SL";
  estimate=0;
  output;
  %do i=1 %to &Number_SharedParamInt.;
  parameter="ASSOCT_SL_&i";
  estimate=0;
  output;
  %end;
 %end;
%if %elementinlist(CUMULATIVE, &SharedParam.) %then %do;
  parameter="ASSOCT_CM";
  estimate=0;
  output;
  %do i=1 %to &Number_SharedParamInt.;
  parameter="ASSOCT_CM_&i";
  estimate=0;
  output;
  %end;
 %end;
%if %elementinlist(COEFFICIENTS, &SharedParam.) %then %do;  
  %let Number_SharedCoefficients = 0;
  %do %while(%qscan(&SharedCoefficients.,&Number_SharedCoefficients+1,%str( )) ne %str());
    %let Number_SharedCoefficients = %eval(&Number_SharedCoefficients+1); 
	%let Reffect&Number_SharedCoefficients. = %scan(&SharedCoefficients.,&Number_SharedCoefficients.);
   %if  NOT (%elementinlist(&&Reffect&Number_SharedCoefficients., BI0 BI1 BI2 BI3 BI4 BI5 BI6 BI7 BI8 BI9)) %then %do;
       %PUT %str(ER)ROR:  RANDOM EFFECTS IN THE SHARED COEFFICIENTS PARAMETER NEED TO BE EITHER bi0 bi1 bi2 bi3 etc;%ABORT;
    %end;
    parameter="ASSOCT_RE_%scan(&SharedCoefficients.,&Number_SharedCoefficients.)"; 
    estimate=0;
    output;
    %do i=1 %to &Number_SharedParamInt.;
     parameter="ASSOCT_RE_%scan(&SharedCoefficients.,&Number_SharedCoefficients.)_&i";
     estimate=0;
     output;
     %end;
  %end;
 %end;
run;
%end;

**** Data Step to rescale parameters to be 1. The following macro variables will be created: 'r_'||parameter ;
data _JMInitialParametersNormalized;
  set %if not (%elementinlist(ONLYEVENT, &AdditionalOptions)) %then %do; _JMsolutionf2 _JMcovparms2 %end;
      %if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) and ((%elementinlist(&EventModel., EXPONENTIAL WEIBULL PIECEWISE)) or &EventCovariates. NE %str() ) %then %do; _JMParameterEstimates2 %end;  
      %if (not (%elementinlist(ONLYLONGI, &AdditionalOptions))) and (%elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE)) and &event_nparam. >1 %then %do; _JMSplinesParam2 %end;
	  %if &SharedParam. NE  NONE %then %do; _JMassoct %end;
  ;
  label parameter = "Parameter";
  label NNestimate = "Not Normalized Estimate";
  label estimate = "Normalized Estimate";

  JMResponse=1;
  JMTimeObs = 1;
  JMDist  =  'DUMMY';
  &SubjectVar. = "DUMMY";
   if abs(estimate)<0.00001 then estimate=0;
   if estimate NE 0 then do;
    call symputx('R_'||parameter,estimate);
	NNestimate=estimate;
    estimate=1;
   end;
   else if estimate = 0 then do;
    call symputx('R_'||parameter, 1);
	NNestimate=0;
   	estimate=0;
   end;
   if estimate=. and  index("&AdditionalOptions.","NOMACROWARN") = 0 then put "WAR" "NING: Some parameters obtained from the disjoint models have missing values"; run;


%skipdisjointmodeling:

%if &InitialParameters. NE DISJOINT %then %do;
**** Adding the variables required to feed NLMIXED and rescaling again the parameters. ; 
data _JMInitialParametersNormalized;
  set &InitialParameters.;
  label parameter = "Parameter";
  label NNestimate = "Not Normalized Estimate";
  label estimate = "Normalized Estimate";
  JMResponse=1;
  JMTimeObs = 1;
  JMDist  =  'DUMMY';
  &SubjectVar. = "DUMMY";
  if estimate NE 0 then do;
    call symputx('R_'||parameter,estimate);
	NNestimate=estimate;
    estimate=1;
   end;
   else if estimate = 0 then do;
    call symputx('R_'||parameter, 1);
	NNestimate=estimate;
   	estimate=0;
   end;
run;
%end;


%if not (%elementinlist(ONLYEVENT, &AdditionalOptions)) %then %do;

********A number of macro variables that will be used later ;

%let Number_SharedParamInt=0; 
%if &SharedParamInt. NE %then %do;
  %do %while(%qscan(&SharedParamInt.,&Number_SharedParamInt+1,%str( )) ne %str());
    %let Number_SharedParamInt = %eval(&Number_SharedParamInt+1);
  %end;
  %do i = 1 %to &Number_SharedParamInt. ;
  	%let SharedParamInt_&i. =  %scan(&SharedParamInt. ,&i.) ;
  %end;
%end;

%let Number_SharedCoefficients = 0;
%do %while(%qscan(&SharedCoefficients.,&Number_SharedCoefficients+1,%str( )) ne %str());
  %let Number_SharedCoefficients = %eval(&Number_SharedCoefficients+1); 
  %let Reffect&Number_SharedCoefficients. = %scan(&SharedCoefficients.,&Number_SharedCoefficients.);
%end;

%let LongiCovariatesTermsNLMIXED = ;  
%if &LongiCovariates. NE %then %do;
  %let Number_LongiCovariates=0; 
  %do %while(%qscan(&LongiCovariates.,&Number_LongiCovariates+1,%str( )) ne %str());
    %let Number_LongiCovariates = %eval(&Number_LongiCovariates+1);
  %end;
  %do i=1 %to &Number_LongiCovariates.;
	%let LongiCovariate_&i. =  %scan(&LongiCovariates. ,&i.) ;
      %let LongiCovariatesTermsNLMIXED =    &LongiCovariatesTermsNLMIXED.    +  &&&&r_l_&&LongiCovariate_&i. * L_&&&&LongiCovariate_&i.*&&LongiCovariate_&i.; 
      %let LongiCovariatesTermsNLMIXED_es_&i. =  &&&&r_l_&&LongiCovariate_&i. * L_&&&&LongiCovariate_&i.;
  %end;
%end;

%let LongiOffsetNLMIXED = ;
%if &LongiOffset. NE %then %let LongiOffsetNLMIXED =  +  &LongiOffset.;; 

%let LonTimIntTermsNLMIXED =    ; 
%let LonTimLagIntTermsNLMIXED = ;
%let LonTimLagIntTermsNLMIXED_kr = ; 
%let LonTimIntTermsNLMIXED_es = ;
%if &LongiTimeInter. NE %then %do;
%let Number_LongiTimeInter=0;
  %do i=1 %to &Number_LongiTimeInters.;
	%let LongiTimeInter_&i. =  %scan(&LongiTimeInter. ,&i.) ;
    %do k = 1 %to %eval(&longi_nparam.-1);
      %let LonTimIntTermsNLMIXED =       &LonTimIntTermsNLMIXED.       +  &&&&r_l_time_b&k._by_&&LongiTimeInter_&i. * L_TIME_B&k._BY_&&LongiTimeInter_&i.*JMTimeObs_b_&k   *&&LongiTimeInter_&i.; 
      %let LonTimLagIntTermsNLMIXED =    &LonTimLagIntTermsNLMIXED.    +  &&&&r_l_time_b&k._by_&&LongiTimeInter_&i. * L_TIME_B&k._BY_&&LongiTimeInter_&i.*JMLagTimeObs_b_&k*&&LongiTimeInter_&i.; 
      %let LonTimLagIntTermsNLMIXED_kr = &LonTimLagIntTermsNLMIXED_kr. +  &&&&r_l_time_b&k._by_&&LongiTimeInter_&i. * L_TIME_B&k._BY_&&LongiTimeInter_&i.*JMTimeObs_kr_b_[r,%eval(&k+1)]*&&LongiTimeInter_&i.; 
      %let LonTimIntTermsNLMIXED_es_&i._&k. =  &&&&r_l_time_b&k._by_&&LongiTimeInter_&i. * L_TIME_B&k._BY_&&LongiTimeInter_&i.;
    %end;
  %end;
%end;
%if &LongiGMatrixParam. = VARCOVAR %then %do;
  %if "&LongiGMatrix." = "VC"  %then %do;
   %do i= %eval(&firsti.) %to %eval(&longi_nparam.-2);
	%do j= &i+1 %to %eval(&longi_nparam.-1);
      %let COV_B&i._B&j._NLMIXED = 0;
    %end;
   %end;
  %end; 
  %else %if "&LongiGMatrix." = "UN"  %then %do;
    %do i= %eval(&firsti.) %to %eval(&longi_nparam.-2);
	  %do j= &i+1 %to %eval(&longi_nparam.-1);
       %let COV_B&i._B&j._NLMIXED = &&r_l_cov_b&i._b&j.*L_COV_B&i._B&j.;
	  %end;
	%end;
  %end;
%end;
%if "&LongiGMatrix." = "ANTE(1)" %then %do;
   %if &firsti. = 0 %then %do;
	%do j= 1 %to %eval(&longi_nparam.-1);
      %let COV_B0_B&j._ANTE = exp((L_LOGSD_INTERCEPT*&r_l_LogSD_Intercept.+L_LOGSD_TIME_B&j.*&&r_l_LogSD_time_b&j.));
	  %do k = 0 %to %eval(&j.-1);
	    %let l = %eval(&k. + 1);
	    %let COV_B0_B&j._ANTE=  &&COV_B0_B&j._ANTE.    *L_RHO_B&k._B&l.*&&r_l_rho_b&k._b&l.;
	  %end;
      %put COV_B0_B&j._ANTE is &&COV_B0_B&j._ANTE.;
    %end;
   %end;
   %do i= 1 %to %eval(&longi_nparam.-2);
	%do j= &i+1 %to %eval(&longi_nparam.-1);
      %let COV_B&i._B&j._ANTE = exp((L_LOGSD_TIME_B&i.*&&r_l_LogSD_time_b&i.+L_LOGSD_TIME_B&j.*&&r_l_LogSD_time_b&j.));
	  %do k = &i. %to %eval(&j.-1);
	    %let l = %eval(&k. + 1);
	    %let COV_B&i._B&j._ANTE=  &&COV_B&i._B&j._ANTE.  *L_RHO_B&k._B&l.*&&r_l_rho_b&k._b&l.;
	  %end;
      %put COV_B&i._B&j._ANTE is &&COV_B&i._B&j._ANTE.;
    %end;
   %end;
%end;

*** Creating link_inv macro with the equation of the inverse of the link function and its derivative;
%if &LongiLink. = IDENTITY %then %do;
  %macro link_inv(x,f_x,df_x_dx);
	%if &f_x. NE %then &f_x. = (&x.);;
	%if &df_x_dx. NE %then &df_x_dx. = 1;;
  %mend link_inv;
%end;
%else %if &LongiLink. = LOGIT %then %do;
  %macro link_inv(x,f_x,df_x_dx);
	%if &f_x. NE %then  &f_x. = exp(&x.)/(1+exp(&x.));;
	%if &df_x_dx. NE %then  &df_x_dx. = exp(&x.)/((1+exp(&x.))**2);;
  %mend link_inv;
%end;
%else %if &LongiLink. = LOG %then %do;
  %macro link_inv(x,f_x,df_x_dx);
	%if &f_x. NE %then   &f_x. = exp(&x.);;
	%if &df_x_dx. NE %then  &df_x_dx. = exp(&x.);;
  %mend link_inv;
%end;
%else %if &LongiLink. = PROBIT %then %do;
  %macro link_inv(x,f_x,df_x_dx);
	%if &f_x. NE %then   &f_x. = PROBNORM(&x.);;
	%if &df_x_dx. NE %then  &df_x_dx. = exp(-0.5*log(2*3.14159265358) - 0.5*((i)**2));;
  %mend link_inv;
%end;

*if not (ONLYEVENT in &AdditionalOptions);%end; 


%if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then %do;
***** Creation of Number_EventCovariates;
%let Number_EventCovariates=0;
%if &EventCovariates. NE  %then %do;
  %do %while(%qscan(&EventCovariates.,&Number_EventCovariates+1,%str( )) ne %str());
    %let Number_EventCovariates = %eval(&Number_EventCovariates+1);
  %end;
  %do i = 1 %to &Number_EventCovariates;
    %let EVENTCovariate_&i. =  %scan(&EventCovariates. ,&i) ;
  %end;
%end;
%let  EVENTCovsNLMIXED =;
%if &EventCovariates. NE  %then %do;
  %do i = 1 %to &Number_EventCovariates.;
    %let  EVENTCovsNLMIXED =    &EVENTCovsNLMIXED.  + &&&&R_E_&&EVENTCovariate_&i..*E_&&EVENTCovariate_&i.*&&EVENTCovariate_&i.; 
    %let  EVENTCovsNLMIXED_ES_&i. =                &&&&R_E_&&EVENTCovariate_&i..*E_&&EVENTCovariate_&i.;
  %end;
%end;
********      Numnber of Subjects and Number of Events    ***********;
 proc sql noprint;
   select count(distinct(&SubjectVar.)), sum(JMEvent) into :Subjects, :Events
   from _JMData where JMResponse NE . and JMDist="EVENT"; 
 quit;

*if not (ONLYLONGI in AdditionalOptions);%end;



***** Creation of N variables for OutputPredictions and OutputPredictData;

  %let Number_OutputPredictions=0;
  %do %while(%qscan(&OutputPredictions.,&Number_OutputPredictions+1,%str( )) ne %str());
    %let Number_OutputPredictions = %eval(&Number_OutputPredictions+1);
  %end;
  %put The user has requested &Number_OutputPredictions predictions dataset;

  %do i = 1 %to &Number_OutputPredictions;
    %let OutputPrediction_&i. =  %scan(&OutputPredictions. ,&i, ' ') ;
  %end;

  %if &OutputPredictData. NE %then %do;
  %let Number_OutputPredictData=0;
  %do %while(%qscan(&OutputPredictData.,&Number_OutputPredictData+1,%str( )) ne %str());
    %let Number_OutputPredictData = %eval(&Number_OutputPredictData+1);
  %end;
  %put The user has provided &Number_OutputPredictData predictions dataset;
  %if &Number_OutputPredictions NE &Number_OutputPredictData %then %do;
     %PUT %str(ER)ROR: The number of predictions datasets requested do not coincide with the number of datasets names provided.; 
   %ABORT; 
  %end;

  %do i = 1 %to &Number_OutputPredictData;
    %let OutputPredictData_&i. =  %scan(&OutputPredictData. ,&i, ' ') ;
  %end;
  %end; 
  %else %do;
    %do i = 1 %to &Number_OutputPredictions;
      %let OutputPredictData_&i. = &&OutputPrediction_&i. ;
    %end;
  %end;


%if NOT (%elementinlist(UNTOUCHEDLISTING, &AdditionalOptions.)) %then %do;
ods listing %if &ListingFile. NE %then file = &ListingFile  ;;
%end;

data _null_; 
%if NOT (%elementinlist(SKIPMACROHEADER, &AdditionalOptions.)) %then %do;
  file print notitles; 
%end;
%if %elementinlist(ONLYEVENT, &AdditionalOptions) %then %do;
 put "Event Model Summary:";
%end;
%else %if %elementinlist(ONLYLONGI, &AdditionalOptions) %then %do;
 put "Longitudinal Model Summary:";
%end;
%else %do;
put "Joint Model Summary:";
%end;

%if not (%elementinlist(ONLYEVENT, &AdditionalOptions)) %then %do;
%if &LongiType. = NORMAL %then 		put "Longitudinal Response Distribution: Normal. Link function: %lowcase(&LongiLink.).";;
%if %elementinlist(&LongiType., BINARY BINARY2) %then 		put "Longitudinal Response Distribution: Binary. Link function: %lowcase(&LongiLink.).";;
%if &LongiType. = BINOMIAL %then 	put "Longitudinal Response Distribution: Binomial. Link function: %lowcase(&LongiLink.).";;
%if &LongiType. = POISSON %then 	put "Longitudinal Response Distribution: Poisson. Link function: %lowcase(&LongiLink.).";;

%if &LongiTimeModel. = LINEAR %then 			put "Longitudinal Process: Random Intercepts and Slopes Model &inttxt.";;
%if &LongiTimeModel. = BSPLINE %then 			put "Longitudinal Process: Random B-splines Model &inttxt.";;
%if &LongiTimeModel. = SPLINE %then 			put "Longitudinal Process: Random Splines Model &inttxt.";;
%if &LongiTimeModel. = NATURALCUBIC %then 		put "Longitudinal Process: Random Natural Cubic Splines Model &inttxt.";;
%end;

%if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then %do;
%if &EventModel. = PIECEWISE %then 			put "Event Process: Piecewise-constant baseline risk function.";;
%if &EventModel. = EXPONENTIAL %then 		put "Event Process: Constant baseline risk function.";;
%if &EventModel. = WEIBULL %then 			put "Event Process: Weibull baseline risk function.";;
%if &EventModel. = GSPLINE %then 			put "Event Process: Splines-generalized Gompertz baseline risk function.";;
%if &EventModel. = GBSPLINE %then 			put "Event Process: B-splines-generalized Gompertz baseline risk function.";;
%if &EventModel. = GNATURALCUBIC %then 		put "Event Process: Natural-cubic-splines-generalized Gompertz baseline risk function.";;
%if &EventModel. = WSPLINE %then 			put "Event Process: Splines-generalized Weibull baseline risk function.";;
%if &EventModel. = WBSPLINE %then 			put "Event Process: B-splines-generalized Weibull baseline risk function.";;
%if &EventModel. = WNATURALCUBIC %then 		put "Event Process: Natural-cubic-splines-generalized Weibull baseline risk function.";;
%if &EventModel. = RPNATURALCUBIC %then 	put "Event Process: (Natural-cubic-splines-based) Royston-Parmar model baseline risk function.";;
%if &EventModel. = RPSPLINE %then 		    put "Event Process: (Splines-based) Royston-Parmar model baseline risk function.";;
%if &EventModel. = RPBSPLINE %then 		    put "Event Process: (B-splines-based) Royston-Parmar model baseline risk function.";;
%end;

put "Parameterization: %lowcase(&SharedParam.).";
%if &SharedLongiTerm.= LINEAR_PREDICTOR %then put "Shared longitudinal term is the non-linked linear predictor";
%if &SharedLagging. >0  			%then 	put "Lagging: &SharedLagging. time units";;
%if ((%elementinlist(CURRENT_VALUE, &SharedParam.)) and (&Transform_current_value_term. = YES))  
                                           %then put "The current value was included in the log-risk function after the following transformation: f(x)=%transform_current_value_term(cvt=x)";

%if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then  %do;
put "Event Process: &subjects. subjects, &events. events.";
%end;
run;

%if NOT (%elementinlist(UNTOUCHEDLISTING, &AdditionalOptions.)) %then %do;
ods listing exclude Parameters ParameterEstimates;
%end;
ods output ParameterEstimates=_JMOutputParameters;
%if %elementinlist(FITSTATISTICS, &AdditionalOptions.) %then %do;
ods output FitStatistics=_JMFitStatistics;
%end;


proc nlmixed data=_JMData   &NLMIXEDOptions.;
parms /data= _JMInitialParametersNormalized;	

%if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then  %do;

  ***** Starting to calcualte the LL of the EVENT entries;
  if JMDist  =  'EVENT' then do;
	  
       /***********************************************************************************/
       /***  Calculating the integral of the hazard from 0 to T using Krorod Rule      ****/
       /***********************************************************************************/
       *15 Variables are expected to exist with the 15 time nodes related to Ti;
       array JMTimeObs_kr_ [15] JMTimeObs_kr_1-JMTimeObs_kr_15; 
       *Variables weight1-weight15 are expected to exist with the 15 weights;
       array JM_kr_weight[15] JM_kr_weight1-JM_kr_weight15;

       %if (%elementinlist(CURRENT_VALUE, &SharedParam.) or %elementinlist(CHANGE_FROM_BASELINE, &SharedParam.)) %then %do;
	   *Variables kbasis1-kbasisM are expected to exist with spline bases for each of the 15 (maybe lagged) time nodes;
       array JMTimeObs_kr_b_[15,&longi_nparam.] JMTimeObs_kr_b_0-JMTimeObs_kr_b_%eval(&longi_nparam.*15-1);  
	   %end;
       %if %elementinlist(SLOPE, &SharedParam.) %then %do;
	   *Variables drv_kbasis1-kbasis75 are expected to exist with the 15 times nodes times the Bspline basis;
       array drv_JMTimeObs_kr_b_[15,&longi_nparam.] drv_JMTimeObs_kr_b_0-drv_JMTimeObs_kr_b_%eval(&longi_nparam.*15-1);  
	   %end;
       %if %elementinlist(CUMULATIVE, &SharedParam.) %then %do;
	   array JMLagTimeObs_kr_ [15] JMLagTimeObs_kr_1-JMLagTimeObs_kr_15; 
       array cm_JMTimeObs_b_[15,%eval(&longi_nparam.-1)] cm_JMTimeObs_b_1-cm_JMTimeObs_b_%eval((&longi_nparam.-1)*15);  
	   %end;
	   %if %elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;
	   *Variables kbasis1-kbasisM are expected to exist with spline bases for each of the 15 (maybe lagged) time nodes;
       array kr_JMEvent_b_[15,&event_nparam.] kr_JMEvent_b_0-kr_JMEvent_b_%eval(&event_nparam.*15-1);  
	   %end;
	   %if %elementinlist(&EventModel., RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;
	   *Variables kbasis1-kbasisM are expected to exist with spline bases for each of the 15 (maybe lagged) time nodes;
       array drv_kr_JMEvent_b_[15,&event_nparam.] drv_kr_JMEvent_b_0-drv_kr_JMEvent_b_%eval(&event_nparam.*15-1);  
	   %end;

       sum=0;
       do r = 1 to 15;

	     
          %if %elementinlist(CURRENT_VALUE, &SharedParam.) %then %do;
			kn_XiBeta_Zibi =    %if &firsti.=0 %then   &r_l_Intercept.*L_INTERCEPT         + bi0;
            %do i = 1 %to   %eval(&longi_nparam.-1);
		        + &&r_l_time_b&i.*L_TIME_B&i.*JMTimeObs_kr_b_[r,%eval(&i.+1)]  + bi&i.*JMTimeObs_kr_b_[r,%eval(&i.+1)] 
            %end;
            &LonTimLagIntTermsNLMIXED_KR. &LongiCovariatesTermsNLMIXED. &LongiOffsetNLMIXED.; 
			%if &SharedLongiTerm. = LINEAR_PREDICTOR %then
             %do;
			  %if &Transform_current_value_term. = NO  %then %let  kn_current_value_term = kn_XiBeta_Zibi;; 
			  %if &Transform_current_value_term. = YES  %then %let  kn_current_value_term = %transform_current_value_term(cvt=kn_XiBeta_Zibi);;
             %end;
            %if &SharedLongiTerm. = MEAN %then %do; 			  
              %link_inv(x=kn_XiBeta_Zibi,f_x=kn_h_XiBeta_Zibi); 
			  %if &Transform_current_value_term. = NO  %then %let  kn_current_value_term = kn_h_XiBeta_Zibi;; 
			  %if &Transform_current_value_term. = YES  %then %let  kn_current_value_term = %transform_current_value_term(cvt=kn_h_XiBeta_Zibi);;
            %end;
          %end;

          %if %elementinlist(BASELINE_VALUE, &SharedParam.)  %then %do;
			kn_Base_XiBeta_Zibi =    %if &firsti.=0 %then   &r_l_Intercept.*L_INTERCEPT         + bi0;
            &LongiCovariatesTermsNLMIXED. &LongiOffsetNLMIXED.; 
			%if &SharedLongiTerm. = LINEAR_PREDICTOR %then
             %do;
			  %let  kn_Base_value_term = kn_Base_XiBeta_Zibi; 
             %end;
            %if &SharedLongiTerm. = MEAN %then %do; 			  
              %link_inv(x=kn_Base_XiBeta_Zibi,f_x=kn_h_Base_XiBeta_Zibi); 
			  %let  kn_Base_value_term = kn_h_Base_XiBeta_Zibi;
            %end;
          %end;


          %if %elementinlist(CHANGE_FROM_BASELINE, &SharedParam.) %then %do;
			kn_Ch_XiBeta_Zibi =   
            %do i = 1 %to   %eval(&longi_nparam.-1);
		        + &&r_l_time_b&i.*L_TIME_B&i.*JMTimeObs_kr_b_[r,%eval(&i.+1)]  + bi&i.*JMTimeObs_kr_b_[r,%eval(&i.+1)] 
            %end;
            &LonTimLagIntTermsNLMIXED_KR.; 
			%if &SharedLongiTerm. = LINEAR_PREDICTOR %then
             %do;
			  %let  kn_chfb_value_term = kn_Ch_XiBeta_Zibi;
             %end;
            %if &SharedLongiTerm. = MEAN %then %do; 			  
              %link_inv(x=kn_Ch_XiBeta_Zibi,f_x=kn_h_Ch_XiBeta_Zibi); 
			  %let  kn_chfb_value_term = kn_h_Ch_XiBeta_Zibi;
            %end;
          %end;

		 %if %elementinlist(SLOPE, &SharedParam.)  %then %do;
		    
		    drv_kn_XiBeta_Zibi = 
           %do i = 1 %to   %eval(&longi_nparam.-1);
	  	      + &&r_l_time_b&i.*L_TIME_B&i.*drv_JMTimeObs_kr_b_[r,%eval(&i.+1)]  + bi&i.*drv_JMTimeObs_kr_b_[r,%eval(&i.+1)]              
		   %end;
           %if &LongiTimeInter. NE %then %do;
             %do i=1 %to &Number_LongiTimeInters.;
		       %do k = 1 %to   %eval(&longi_nparam.-1);
                   +  &&&&r_l_time_b&k._by_&&LongiTimeInter_&i.*L_TIME_B&k._BY_&&LongiTimeInter_&i.*&&LongiTimeInter_&i.*drv_JMTimeObs_kr_b_[r,%eval(&k.+1)] 
		       %end;
		     %end;
		   %end;
 		   ;
		   %if &SharedLongiTerm. = LINEAR_PREDICTOR %then %let kn_slope_term =drv_kn_XiBeta_Zibi;;
           %if &SharedLongiTerm. = MEAN %then %do;
            %link_inv(x=kn_XiBeta_Zibi,df_x_dx=kn_dh_dt_XiBeta_Zibi);
			%let kn_slope_term =kn_dh_dt_XiBeta_Zibi*drv_kn_XiBeta_Zibi;
           %end;
		 %end;



		 %if %elementinlist(CUMULATIVE, &SharedParam.)  %then %do;
		    cm_kn_XiBeta_Zibi = %if &firsti.=0 %then (&r_l_Intercept.*L_INTERCEPT   + bi0)*JMLagTimeObs_kr_[r];
           %do i = 1 %to   %eval(&longi_nparam.-1);
	  	      + (&&r_l_time_b&i.*L_TIME_B&i. + bi&i.)*cm_JMTimeObs_b_[r,&i]
		   %end;
         %if &LongiTimeInter. NE %then %do;
           %do i=1 %to &Number_LongiTimeInters.;
		   %do k = 1 %to   %eval(&longi_nparam.-1);
                   +  &&&&r_l_time_b&k._by_&&LongiTimeInter_&i.*L_TIME_B&k._BY_&&LongiTimeInter_&i.*&&LongiTimeInter_&i.*cm_JMTimeObs_b_[r,&k.]
		   %end;
		   %end;
		 %end;
           %if &LongiCovariates. NE %then %do;
			+ (&LongiCovariatesTermsNLMIXED.)*JMLagTimeObs_kr_[r] 
		   %end;

           %if &LongiOffset.  NE %then %do;
			+ (&LongiOffsetNLMIXED.)*JMLagTimeObs_kr_[r] 
		   %end;
		   ;
		 %end;

		 ********Baseline Hazard Fuction of the KronRod Rule Nodes;
	     %if &EventModel. = WEIBULL  %then %do;
		      %if &JMStrataN. = 1 %then %do;
				E_SHAPE = exp(E_LOGSHAPE*&r_e_LogShape.);
		 		kn_logbh =    (E_LOGSHAPE*&r_e_LogShape.) - E_SHAPE * (E_LOGSCALE*&r_e_LogScale.) + (E_SHAPE - 1) * log(JMTimeObs_kr_[r]) ;
   	          %end;
		      %else %if &JMStrataN. > 1 %then %do;
		        %do s=1 %to  &JMStrataN.;
				   E_SHAPE_S&s. = exp(E_LOGSHAPE_S&s.*&&r_e_LogShape_S&s.);
		 		   if JMStrataN=&s. then  kn_logbh =    (E_LOGSHAPE_S&s.*&&r_e_LogShape_S&s.) - E_SHAPE_S&s. * (E_LOGSCALE_S&s.*&&r_e_LogScale_S&s.) + (E_SHAPE_S&s. - 1) * log(JMTimeObs_kr_[r]) ;
				%end;
			  %end;
		 %end;
		 %else %if &EventModel. = EXPONENTIAL %then %do;
		    %if &JMStrataN. = 1 %then %do;
                kn_logbh = E_LOGHAZARD*&r_E_LogHazard.;    
            %end;
		    %else %if &JMStrataN. > 1 %then %do;
		        %do s=1 %to  &JMStrataN.; 
                       if JMStrataN=&s. then  kn_logbh = E_LOGHAZARD_S&s.*&&r_E_LogHazard_S&s.; 
				%end;
			  %end;
		 %end;
		 %else %if %elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC) %then %do;
		   %if &JMStrataN. = 1 %then %do;
	        kn_logbh =       %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i.*E_TIME_B&i.*kr_JMEvent_b_[r,%eval(&i.+1)]  %end;  ;
		   %end;
		   %else %if &JMStrataN. > 1 %then %do;
			 %do s=1 %to  &JMStrataN.;
			 if JMStrataN=&s. then kn_logbh =  %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i._s&s..*E_TIME_B&i._S&s.*kr_JMEvent_b_[r,%eval(&i.+1)]  %end; ; 
			 %end;
		   %end;
		 %end;
		 %else %if %elementinlist(&EventModel., RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;
		   %if &JMStrataN. = 1 %then %do;
	        kn_logbh =   -log(JMTimeObs_kr_[r])  %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i.*E_TIME_B&i.*kr_JMEvent_b_[r,%eval(&i.+1)]  %end;   
			                      + log( %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i.*E_TIME_B&i.*drv_kr_JMEvent_b_[r,%eval(&i.+1)]  %end; )
            ;
  		    
		   %end;
		   %else %if &JMStrataN. > 1 %then %do;
			 %do s=1 %to  &JMStrataN.;
			 if JMStrataN=&s. then kn_logbh =   -log(JMTimeObs_kr_[r])  %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i._s&s..*E_TIME_B&i._S&s.*kr_JMEvent_b_[r,%eval(&i.+1)]  %end;   
			                      + log( %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i._s&s..*E_TIME_B&i._S&s.*drv_kr_JMEvent_b_[r,%eval(&i.+1)]  %end; )
            ;
			 %end;
		   %end;
		 %end;
		 %else %if &EventModel. = PIECEWISE %then %do; 
	        %if &JMStrataN. = 1 %then %do;
             if               JMTimeObs_kr_[r] <= input("&Eventknot1.",hex16.)  then kn_logbh =     E_LOGHAZARD_1*&r_e_LogHazard_1.;  
             %do i = 1 %to &EventNknots.;
		        %let j= %eval(&i.+1);
		        else if input("&&Eventknot&i.",hex16.) < JMTimeObs_kr_[r] <= input("&&Eventknot&j.",hex16.)   then kn_logbh =     E_LOGHAZARD_&j. * &&r_e_LogHazard_&j.;         
		     %end;
			%end;
			%else %if &JMStrataN. > 1 %then %do;
			 %do s=1 %to  &JMStrataN.;
			       %if &s. >1 %then else;     
                   if      JMTimeObs_kr_[r] <= input("&Eventknot1.",hex16.) and JMStrataN=&s. then kn_logbh =     E_LOGHAZARD_1_S&s.*&&r_e_LogHazard_1_s&s..;  
			 %end;
             %do s=1 %to  &JMStrataN.;
               %do i = 1 %to &EventNknots.;
		         %let j= %eval(&i.+1);
		         %let condition = else if input("&&Eventknot&i.",hex16.) < JMTimeObs_kr_[r] <= input("&&Eventknot&j.",hex16.) and JMStrataN=&s.  then kn_logbh =     E_LOGHAZARD_&j._S&s. * &&r_e_LogHazard_&j._s&s.;         
		         &condition ;
		       %end;
			 %end;
			%end;
		     else abort;
         %end;
 
          kn_logh =      kn_logbh    &EVENTCovsNLMIXED.  
		  	  %if %elementinlist(CURRENT_VALUE, &SharedParam. ) %then %do;
                 + &r_assoct_td.*ASSOCT_TD*&kn_current_value_term.
				   %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_td_&i.*ASSOCT_TD_&i.*&kn_current_value_term.*&&SharedParamInt_&i. 
				   %end;
              %end;
		  	  %if %elementinlist(BASELINE_VALUE, &SharedParam. ) %then %do;
                 + &r_assoct_bs.*ASSOCT_BS*(&kn_Base_value_term.)
				   %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_bs_&i.*ASSOCT_BS_&i.*(&kn_Base_value_term.)*&&SharedParamInt_&i. 
				   %end;

              %end;
		  	  %if %elementinlist(CHANGE_FROM_BASELINE, &SharedParam. ) %then %do;
                 + &r_assoct_ch.*ASSOCT_CH*(&kn_chfb_value_term.)
				   %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_ch_&i.*ASSOCT_CH_&i.*(&kn_chfb_value_term.)*&&SharedParamInt_&i. 
				   %end;
              %end;
	          %if %elementinlist(SLOPE, &SharedParam.)  %then %do;
                + &r_assoct_sl.*ASSOCT_SL*&kn_slope_term.
				   %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_sl_&i.*ASSOCT_SL_&i.*&kn_slope_term.*&&SharedParamInt_&i. 
				   %end;
              %end;
	          %if %elementinlist(CUMULATIVE, &SharedParam.)  %then %do;
                + &r_assoct_cm.*ASSOCT_CM*cm_kn_XiBeta_Zibi
				   %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_cm_&i.*ASSOCT_CM_&i.*cm_kn_XiBeta_Zibi*&&SharedParamInt_&i. 
				   %end;
              %end;
			  %if %elementinlist(COEFFICIENTS, &SharedParam.)  %then %do;
			   %macro addreffects;
                %do nr=1 %to &Number_SharedCoefficients.;
				   %let what= &&Reffect&nr.;
                   + &&r_assoct_re_&what. * ASSOCT_RE_&what. * &what.
				   %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_re_&what._&i.*ASSOCT_RE_&what._&i.*&what.*&&SharedParamInt_&i. 
				   %end;
				%end;     
				%mend addreffects;
				%addreffects;
              %end;
           ;

  	     sum = sum + exp(kn_logh)*JM_kr_weight[r]; 
       end;
       cum_hi=sum*(JMResponse-JMFromTimeObs)*0.5; 
       /***********************************************************************************/;
	   /* Cum_hi is the integral of the hazard from 0 to T estimated using the Krorod Rule */
       /***********************************************************************************/;




       /***********************************************************************************/;
	   /* The log-likehood for censored observations                                      */
       /***********************************************************************************/;

	      %if %elementinlist(CURRENT_VALUE, &SharedParam.) %then %do;
		  ***** For the EVENT model we need to evaluate h-1(mi()) at the (maybe lagged) times JMLagTimeObs; 
	      lag_XiBeta_Zibi =  %if &firsti.=0 %then &r_l_Intercept.*L_INTERCEPT   + bi0; 

           %do i = 1 %to   %eval(&longi_nparam.-1);
		      %let mi_effect = + &&r_l_time_b&i.*L_TIME_B&i.*JMLagTimeObs_b_&i. + bi&i.*JMLagTimeObs_b_&i.;
               &mi_effect. 
		      %end;
            &LonTimLagIntTermsNLMIXED.  &LongiCovariatesTermsNLMIXED.  &LongiOffsetNLMIXED.
          ;

		  %if &SharedLongiTerm. = LINEAR_PREDICTOR %then 
		     %do;
			  %if &Transform_current_value_term. = NO %then %let  current_value_term =lag_XiBeta_Zibi;;
			  %if &Transform_current_value_term. = YES %then  %let  current_value_term = %transform_current_value_term(cvt=lag_XiBeta_Zibi);;
             %end;
          %if &SharedLongiTerm. = MEAN %then %do; 
              %link_inv(x=lag_XiBeta_Zibi,f_x=h_lag_XiBeta_Zibi);
			  %if &Transform_current_value_term. = NO %then  %let current_value_term = h_lag_XiBeta_Zibi;;
			  %if &Transform_current_value_term. = YES %then %let  current_value_term = %transform_current_value_term(cvt=h_lag_XiBeta_Zibi);;
             %end;
		 %end;

	      %if %elementinlist(BASELINE_VALUE, &SharedParam.) %then %do;
		  ***** For the EVENT model we need to evaluate h-1(mi()) at the (maybe lagged) times JMLagTimeObs; 
	      Base_XiBeta_Zibi =  %if &firsti.=0 %then &r_l_Intercept.*L_INTERCEPT   + bi0; 
             &LongiCovariatesTermsNLMIXED.  &LongiOffsetNLMIXED.
          ;
		  %if &SharedLongiTerm. = LINEAR_PREDICTOR %then 
		     %do;
			  %let  base_value_term =Base_XiBeta_Zibi;
             %end;
          %if &SharedLongiTerm. = MEAN %then %do; 
              %link_inv(x=Base_XiBeta_Zibi,f_x=h_Base_XiBeta_Zibi);
			  %let base_value_term = h_Base_XiBeta_Zibi;
             %end;
		 %end;


	      %if %elementinlist(CHANGE_FROM_BASELINE, &SharedParam.) %then %do;
		  ***** For the EVENT model we need to evaluate h-1(mi()) at the (maybe lagged) times JMLagTimeObs; 
	      Ch_XiBeta_Zibi =  
           %do i = 1 %to   %eval(&longi_nparam.-1);
		         + &&r_l_time_b&i.*L_TIME_B&i.*JMLagTimeObs_b_&i. + bi&i.*JMLagTimeObs_b_&i.
  	       %end;
            &LonTimLagIntTermsNLMIXED.  
          ;
		  %if &SharedLongiTerm. = LINEAR_PREDICTOR %then 
		     %do;
			  %let  chfb_value_term =Ch_XiBeta_Zibi;
             %end;
          %if &SharedLongiTerm. = MEAN %then %do; 
              %link_inv(x=Ch_XiBeta_Zibi,f_x=h_Ch_XiBeta_Zibi);
			  %let chfb_value_term = h_Ch_XiBeta_Zibi;
             %end;
		 %end;


	     %if %elementinlist(SLOPE, &SharedParam.) %then %do;
		   drv_XiBeta_Zibi = 
            %do i = 1 %to   %eval(&longi_nparam.-1);
		       + &&r_l_time_b&i.*L_TIME_B&i.*drv_JMLagTimeObs_b_&i. + bi&i.*drv_JMLagTimeObs_b_&i.
            %end;
			%if &LongiTimeInter. NE %then %do;
             %do i=1 %to &Number_LongiTimeInters.;
			   %do k = 1 %to   %eval(&longi_nparam.-1);
                    +  &&&&r_l_time_b&k._by_&&LongiTimeInter_&i.*L_TIME_B&k._BY_&&LongiTimeInter_&i.*&&LongiTimeInter_&i.*drv_JMLagTimeObs_b_&k.
		       %end;
  			 %end;
           %end;

		   ;
		   %if &SharedLongiTerm. = LINEAR_PREDICTOR %then %let slope_term =drv_XiBeta_Zibi;;
		   %if &SharedLongiTerm. = MEAN %then %do;
		     %link_inv(x=lag_XiBeta_Zibi,df_x_dx=dh_dt_lag_XiBeta_Zibi);
			 %let slope_term = (dh_dt_lag_XiBeta_Zibi*drv_XiBeta_Zibi);
           %end;
		 %end;

	     %if %elementinlist(CUMULATIVE, &SharedParam.) %then %do;
		   cm_XiBeta_Zibi =  %if &firsti.=0 %then (&r_l_Intercept.*L_INTERCEPT   + bi0)*JMLagTimeObs 
            %do i = 1 %to   %eval(&longi_nparam.-1);
		       + (&&r_l_time_b&i.*L_TIME_B&i. + bi&i.)*cm_Bin&i.
		    %end;
			%if &LongiTimeInter. NE %then %do;
             %do i=1 %to &Number_LongiTimeInters.;
			   %do k = 1 %to   %eval(&longi_nparam.-1);
                    +  &&&&r_l_time_b&k._by_&&LongiTimeInter_&i.*L_TIME_B&k._BY_&&LongiTimeInter_&i.*&&LongiTimeInter_&i.*cm_Bin&k.
		       %end;
  			 %end;
           %end;
           %if &LongiCovariates. NE %then %do;
			  + (&LongiCovariatesTermsNLMIXED.)*JMLagTimeObs 
		   %end;
           %if &LongiOffset.  NE %then %do;
			+ (&LongiOffsetNLMIXED.)*JMLagTimeObs
		   %end;
		   ;
		 %end;
       ********Baseline Hazard Fuction at the observed time T;


	     %if &EventModel. = WEIBULL %then %do;
		  %if &JMStrataN. = 1 %then %do;
            log_bhi = (E_LOGSHAPE*&r_e_LogShape.) - E_SHAPE * (E_LOGSCALE*&r_e_LogScale.) + (E_SHAPE - 1)*log(JMResponse) ; 
			%end;
		   %else %if &JMStrataN. > 1 %then %do;
		     %do s=1 %to  &JMStrataN.;
			   if JMStrataN=&s. then log_bhi = (E_LOGSHAPE_S&s.*&&r_e_LogShape_s&s.) - E_SHAPE_S&s. * (E_LOGSCALE_S&s.*&&r_e_LogScale_s&s.) + (E_SHAPE_S&s. - 1)*log(JMResponse); 
			 %end;
		   %end;
		 %end;
		 %else %if &EventModel. = EXPONENTIAL %then %do;
		    %if &JMStrataN. = 1 %then %do;
	          log_bhi = E_LOGHAZARD*&r_e_LogHazard.; 
			%end;
		    %else %if &JMStrataN. > 1 %then %do;
		     %do s=1 %to  &JMStrataN.;
			   if JMStrataN=&s. then log_bhi = E_LOGHAZARD_S&s*&&r_e_LogHazard_S&s.;
			 %end;
		   %end;
		 %end;
         %else %if %elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC) %then %do;
		   %if &JMStrataN. = 1 %then %do;
	        log_bhi =    %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i.*E_TIME_B&i.*JMEvent_b_&i.  %end;   ;
		   %end;
		   %else %if &JMStrataN. > 1 %then %do;
			 %do s=1 %to  &JMStrataN.;
			 if JMStrataN=&s. then log_bhi =     %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i._s&s.*E_TIME_B&i._S&s.*JMEvent_b_&i.  %end;; 
			 %end;
		   %end;
		 %end;
		 %else %if %elementinlist(&EventModel., RPNATURALCUBIC RPSPLINE RPBSPLINE) %then %do;
		   %if &JMStrataN. = 1 %then %do;
            log_bhi =   -log(JMResponse)   %do i = 0 %to   %eval(&event_nparam.-1);       + &&r_e_time_b&i.*E_TIME_B&i.*JMEvent_b_&i.     %end;   
			                               + log( %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i.*E_TIME_B&i.*drv_JMEvent_b_&i.  %end; )
            ;

		   %end;
		   %else %if &JMStrataN. > 1 %then %do;
			 %do s=1 %to  &JMStrataN.;
			 if JMStrataN=&s. then log_bhi =  -log(JMResponse)   %do i = 0 %to   %eval(&event_nparam.-1);       + &&r_e_time_b&i._s&s.*E_TIME_B&i._S&s.*JMEvent_b_&i.     %end;   
			                               + log( %do i = 0 %to   %eval(&event_nparam.-1);  + &&r_e_time_b&i._s&s.*E_TIME_B&i._S&s.*drv_JMEvent_b_&i.  %end; )
            ;
			 %end;
		   %end;
		 %end;
		 %else %if &EventModel. = PIECEWISE %then %do; 
		 %if &JMStrataN. = 1 %then %do;
	 	        if               JMResponse <= input("&Eventknot1.",hex16.)  then log_bhi =     E_LOGHAZARD_1*&r_e_LogHazard_1.;          
		        %do i = 1 %to &EventNknots.;
		          %let j= %eval(&i.+1);
		          %let condition =  else if input("&&Eventknot&i.",hex16.) < JMResponse <= input("&&Eventknot&j.",hex16.)  then log_bhi =     E_LOGHAZARD_&j.*&&r_e_LogHazard_&j.;          
		          &condition ;
		        %end;
		  %end;
		  %else %if &JMStrataN. > 1 %then %do;
			 %do s=1 %to  &JMStrataN.;
			       %if &s. >1 %then else;     
                   if      JMResponse <= input("&Eventknot1.",hex16.) and JMStrataN=&s. then log_bhi =     E_LOGHAZARD_1_S&s.*&&r_e_LogHazard_1_s&s..;  
			 %end;
             %do s=1 %to  &JMStrataN.;
               %do i = 1 %to &EventNknots.;
		         %let j= %eval(&i.+1);
		         %let condition = else if input("&&Eventknot&i.",hex16.) < JMResponse <= input("&&Eventknot&j.",hex16.) and JMStrataN=&s.  then log_bhi =     E_LOGHAZARD_&j._S&s. * &&r_e_LogHazard_&j._s&s.;         
		         &condition ;
		       %end;
			 %end;
		  %end;
		  else abort;
	      %end;

       log_hi =    log_bhi   &EVENTCovsNLMIXED. 
   	               %if %elementinlist(CURRENT_VALUE, &SharedParam.) %then %do;
	                  + &r_assoct_td.*assoct_td * &current_value_term.
				      %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_td_&i.*ASSOCT_TD_&i.*&current_value_term. * &&SharedParamInt_&i. 
  				     %end;
                   %end;
		  	  %if %elementinlist(BASELINE_VALUE, &SharedParam. ) %then %do;
                 + &r_assoct_bs.*ASSOCT_BS*(&Base_value_term.)
				   %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_bs_&i.*ASSOCT_BS_&i.*(&Base_value_term.)*&&SharedParamInt_&i. 
				   %end;

              %end;
		  	  %if %elementinlist(CHANGE_FROM_BASELINE, &SharedParam. ) %then %do;
                 + &r_assoct_ch.*ASSOCT_CH*(&chfb_value_term.)
				   %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_ch_&i.*ASSOCT_CH_&i.*(&chfb_value_term.)*&&SharedParamInt_&i. 
				   %end;
              %end;

   	               %if %elementinlist(SLOPE, &SharedParam.) %then %do;
	                  + &r_assoct_sl.*assoct_sl * &slope_term. 
					 %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_sl_&i.*ASSOCT_SL_&i.* &slope_term. * &&SharedParamInt_&i. 
  				     %end;
                    %end;
   	               %if %elementinlist(CUMULATIVE, &SharedParam.) %then %do;
	                  + &r_assoct_cm.*assoct_cm * cm_XiBeta_Zibi 
					 %do i=1 %to &Number_SharedParamInt.;
                        + &&r_assoct_cm_&i.*ASSOCT_cm_&i.* cm_XiBeta_Zibi * &&SharedParamInt_&i. 
  				     %end;
                    %end;


					%if %elementinlist(COEFFICIENTS, &SharedParam.) %then %do;
					%addreffects;
					%end;
         ;

       /***********************************************************************************/;
	   /* And finally the log-likehood is given below                                     */
       /***********************************************************************************/;
       if JMEvent =0 then ll =  - cum_hi;  
       else if JMEvent = 1 then ll =  log_hi - cum_hi;
    end;else  
	%end; 

	%if not (%elementinlist(ONLYEVENT, &AdditionalOptions)) %then  %do;
    if JMDist  =  'LONGI' then do;

	     XiBeta =  %if &firsti.=0 %then   &r_l_Intercept.*L_INTERCEPT ; 

              %do i = 1 %to   %eval(&longi_nparam.-1);
		         + &&r_l_time_b&i.*L_TIME_B&i.*JMTimeObs_b_&i.
		      %end;
              &LonTimIntTermsNLMIXED.  &LongiCovariatesTermsNLMIXED. &LongiOffsetNLMIXED.;
         XiBeta_Zibi =    XiBeta  %if &firsti.=0 %then + bi0 ;
              %do i = 1 %to   %eval(&longi_nparam.-1);
		         + bi&i.*JMTimeObs_b_&i.
		      %end;
             ;

		%link_inv(x=XiBeta_Zibi, f_x=mi);

  	 %if &LongiType. = NORMAL %then %do;
        ll = -0.5*log(2*3.14159265358) - (L_LOGSD_RESIDUAL*&r_l_LogSD_Residual. ) - 0.5*((JMResponse  -   mi)**2)/(exp(2*(L_LOGSD_RESIDUAL*&r_l_LogSD_Residual.)));
     %end;
     %else %if &LongiType. = BINARY %then %do;
        ll=(JMResponse = &LongiEventCat.)*log(mi)+ (JMResponse NE . and JMResponse NE &LongiEventCat.)*log(1-mi);
	 %end;
     %else %if &LongiType. = BINARY2 %then %do;
		if JMResponse = &LongiEventCat. and mi > 1e-8 then ll = log(mi);
		else if JMResponse = &LongiEventCat. and .<mi<=1e-8 then ll = -1e100;
		else if JMResponse NE . and JMResponse NE &LongiEventCat. and 1-mi > 1e-8 then ll = log(1-mi);
		else if JMResponse NE . and JMResponse NE &LongiEventCat. and .<1-mi<=1e-8 then ll = -1e100;
     %end;
     %else %if &LongiType. = BINOMIAL %then %do;
        ll = JMResponse*log(mi)+(&LongiTrialsVar.-JMResponse)*log(1-mi)+log(gamma(&LongiTrialsVar.+1))-log(gamma(JMResponse+1))-log(gamma(&LongiTrialsVar. - JMResponse+1)); 
     %end;
	 %else %if &LongiType. = POISSON %then %do;
	     ll = JMResponse*log(mi) - mi - log(gamma(JMResponse+1)); 
     %end;
   end;else 
   %end;
   ll=0;

*Section below updated in version 1.08 to avoid %SYSMACEXIST and make the macro functionable before SAS 9.3;
%if &nlmixedexpression = YES %then %do;
    %nlmixedexpression;
%end;

   model JMResponse ~ general(ll);
 
   %if not (%elementinlist(ONLYEVENT, &AdditionalOptions)) %then  %do;
   random %if &firsti.=0 %then bi0;    %do i = 1 %to %eval(&longi_nparam.-1); bi&i. %end;
          ~ normal(  [%if &firsti.=0 %then 0,; %do i = 1 %to %eval(&longi_nparam.-2); 0, %end;   0],

   %if &LongiGMatrixParam. = VARCOVAR %then %do;

		              [%if &firsti.=0 %then                 exp(2*(L_LOGSD_INTERCEPT*&r_l_LogSD_Intercept.)), ;

						%do j= 1 %to %eval(&longi_nparam.-1);
							%do i= &firsti. %to &j.-1;
  						             %let AddCov = &&Cov_b&i._b&j._NLMIXED.; 
                                     &AddCov. ,
							%end;
							
						    %let AddSD =  exp(2*(L_LOGSD_TIME_B&j.*&&r_l_LogSD_time_b&j.));
							&AddSD.  %if &j. < %eval(&longi_nparam.-1) %then ,; 
					    %end;

                       ])   sub=&SubjectVar.;

  %end;
  %else %if &LongiGMatrixParam. = CHOL and  "&LongiGMatrix." = "UN" %then %do; 
					   	              [ 
						%do i = 1  %to  %eval(&longi_nparam.-&firsti.);
						  %do j = 1  %to  &i.;
						    %let element= (L_GCHOL_B_&i._1*&&r_l_gchol_b_&i._1)*(L_GCHOL_B_&j._1*&&r_l_gchol_b_&j._1);
						    %do k =2 %to &j.;
						      %let  element  =  &element.  + (L_GCHOL_B_&i._&k. * &&r_l_gchol_b_&i._&k.)*(L_GCHOL_B_&j._&k. * &&r_l_gchol_b_&j._&k.);
							%end;
                              &element. 
							%if &i NE  %eval(&longi_nparam.-&firsti.) or &j NE  %eval(&longi_nparam.-&firsti.) %then,;
                          %end;
						%end;
                       ])   sub=&SubjectVar.;

  %end; 
  %else %if &LongiGMatrixParam. = CHOL and  "&LongiGMatrix." = "VC" %then %do; 
					   	              [ 
						%do i = 1  %to %eval(&longi_nparam.-&firsti.);
						  %do j = 1  %to  &i.;
						    %if &i. = &j. %then %do; 
						      %let element= (L_GCHOL_B_&i._&i.*&&r_l_gchol_b_&i._&i.)*(L_GCHOL_B_&i._&i.*&&r_l_gchol_b_&i._&i.);
                              &element. 
                             %end;
                            %else %do; 0 %end;
							%if &i NE  %eval(&longi_nparam.-&firsti.) or &j NE %eval(&longi_nparam.-&firsti.) %then,;
                          %end;
						%end;
                       ])   sub=&SubjectVar.;

  %end; 
 %else %if "&LongiGMatrix." = "ANTE(1)" %then %do; 

		              [%if &firsti.=0 %then  exp(2*(L_LOGSD_INTERCEPT*&r_l_LogSD_Intercept.)), ;

						%do j= 1 %to %eval(&longi_nparam.-1);
							%do i= &firsti. %to &j.-1;
  						             &&Cov_b&i._b&j._ANTE.,
							%end;
							 exp(2*(L_LOGSD_TIME_B&j.*&&r_l_LogSD_time_b&j.)) %if &j. < %eval(&longi_nparam.-1) %then ,; 
					    %end;
                       ])   sub=&SubjectVar.;

 %end;
 %else %do; %PUT %str(ER)ROR: LongiGMatrixParam MUST = CHOL or VARCOVAR; %abort; %end;
  
   %if &firsti.=0 %then estimate 'Longitudinal Process: Intercept' &r_l_Intercept.*L_INTERCEPT;;
   %if &LongiTimeModel. = LINEAR %then %do;
     estimate 'Longitudinal Process: Slope' &r_l_time_b1.*L_TIME_B1;
   %end;
   %else %if &LongiTimeModel. = BSPLINE or &LongiTimeModel. = SPLINE or &LongiTimeModel. = NATURALCUBIC %then %do;
     %do i = 1 %to %eval(&longi_nparam.-1);
	   %let estimate_sta = estimate "Longitudinal Process: Time Parameter &i" L_TIME_B&i*&&r_l_time_b&i.;
       &estimate_sta.;
	 %end; 
   %end;
   %if &LongiTimeInter. NE %then %do;
     %do i = 1 %to &Number_LongiTimeInters.;
           %if &LongiTimeModel. = LINEAR %then %do;
	          %do k = 1 %to %eval(&longi_nparam.-1);
                estimate "Longitudinal Process: &&LongiTimeInter_&i. by slope" &&LonTimIntTermsNLMIXED_es_&i._&k.;
	          %end;
           %end;
       %else %if &LongiTimeModel. = BSPLINE or &LongiTimeModel. = SPLINE or &LongiTimeModel. = NATURALCUBIC %then %do;
	     %do k = 1 %to %eval(&longi_nparam.-1);
           estimate "Longitudinal Process: &&LongiTimeInter_&i. by Time Parameter &k" &&LonTimIntTermsNLMIXED_es_&i._&k.;
	     %end;
	   %end;
	 %end;
   %end;

   %if &LongiCovariates. NE %then %do;
     %do i = 1 %to &Number_LongiCovariates.;
      estimate "Longitudinal Process: &&LongiCovariate_&i." &&LongiCovariatesTermsNLMIXED_es_&i.;
     %end;
   %end;
   %if &LongiType. = NORMAL %then %do;
      estimate 'Longitudinal Process: Log SD Residual' L_LOGSD_RESIDUAL*&r_l_LogSD_Residual.;
   %end;
  
   %if &LongiGMatrixParam. = VARCOVAR or &LongiGMatrixParam. = VARCORR %then %do;
     %if &firsti.=0 %then estimate 'Longitudinal Process: Log SD Random Intercept' L_LOGSD_INTERCEPT*&r_l_LogSD_Intercept.;;
     %if &LongiTimeModel. = LINEAR %then %do;
        estimate 'Longitudinal Process: Log SD Random Slope' L_LOGSD_TIME_B1*&r_l_LogSD_time_b1.;
     %end; 
     %if &LongiTimeModel. = BSPLINE or &LongiTimeModel. = SPLINE or &LongiTimeModel. = NATURALCUBIC %then %do;
        %do i = 1 %to %eval(&longi_nparam.-1);
	       %let estimate_sta = estimate "Longitudinal Process: Log SD Random Time Parameter &i" L_LOGSD_TIME_B&i*&&r_l_LogSD_time_b&i.;
           &estimate_sta.;
	    %end; 
     %end;
     %if "&LongiGMatrix" = "ANTE(1)" %then %do;
       %if &LongiTimeModel. = LINEAR %then %do;
          %if &firsti.=0 %then estimate 'Longitudinal Process: Correlation (Intercept, Slope)' L_RHO_B0_B1*&&r_l_rho_b0_b1;;
	    %end; 
       %if &LongiTimeModel. = BSPLINE or &LongiTimeModel. = SPLINE or &LongiTimeModel. = NATURALCUBIC %then %do;
		   %do i= &firsti. %to %eval(&longi_nparam.-2);
		     %let j = %eval(&i.+1);
             estimate "Longitudinal Process: Correlation Time Parameter (&i., &j.)" L_RHO_B&i._B&j.*&&r_l_rho_b&i._b&j.;
           %end;  
	    %end; 
     %end;
     %if "&LongiGMatrix" = "UN" %then %do;
       %if &LongiTimeModel. = LINEAR %then %do;
          %if &firsti.=0 %then estimate 'Longitudinal Process: Covariance Intercept by Slope' &Cov_b0_b1_NLMIXED.;;
	    %end; 
       %if &LongiTimeModel. = BSPLINE or &LongiTimeModel. = SPLINE or &LongiTimeModel. = NATURALCUBIC %then %do;
	       %do j = 1 %to %eval(&longi_nparam.-1);
		   %do i= &firsti. %to &j.-1;
             estimate "Longitudinal Process: Covariance Time Param &i. by Param &j." &&Cov_b&i._b&j._NLMIXED.;
           %end; 
           %end;  
	    %end; 
     %end;
   %end;
   %else %if &LongiGMatrixParam. = CHOL and "&LongiGMatrix." = "UN" %then %do; 
						%do i = 1  %to  %eval(&longi_nparam.-&firsti.);
						  %do j = 1  %to  &i.;
                               estimate "Longitudinal Process: G (Chol) Matrix (%eval(&i.-&one_m_firsti.),%eval(&j.-&one_m_firsti.))"  L_GCHOL_B_&i._&j. * &&r_l_gchol_b_&i._&j.;
                          %end;
						%end;
    %end; 
   %else %if &LongiGMatrixParam. = CHOL and "&LongiGMatrix." = "VC" %then %do; 
						%do i = 1  %to  %eval(&longi_nparam.-&firsti.);
                            estimate "Longitudinal Process: G (Chol) Matrix (%eval(&i.-&one_m_firsti.),%eval(&i.-&one_m_firsti.))"  L_GCHOL_B_&i._&i.*&&r_l_gchol_b_&i._&i.;
						%end;
    %end;
   %end; 
   %if not (%elementinlist(ONLYLONGI, &AdditionalOptions)) %then  %do;

   %if &EventModel. = WEIBULL %then %do;
     %if &JMStrataN. = 1 %then %do;
      estimate 'Event Process: Weibull Log Shape' E_LOGSHAPE * &r_e_LogShape.;
      estimate 'Event Process: Weibull Log Scale' E_LOGSCALE * &r_e_LogScale.;
	 %end;
	 %else %if &JMStrataN. > 1 %then %do;
     %do s=1 %to  &JMStrataN.;
      estimate "Event Process: Stratum &s. Weibull Log Shape" E_LOGSHAPE_S&s. * &&r_e_LogShape_S&s.;
      estimate "Event Process: Stratum &s. Weibull Log Scale" E_LOGSCALE_S&s. * &&r_e_LogScale_S&s.;
     %end;
	 %end;
   %end;
   %if &EventModel. =  EXPONENTIAL  %then %do;
     %if &JMStrataN. = 1 %then %do;
      estimate 'Event Process: Log Hazard'  E_LOGHAZARD*&r_e_LogHazard.;
     %end;
	 %else %if &JMStrataN. > 1 %then %do;
     %do s=1 %to  &JMStrataN.;
       estimate "Event Process: Stratum &s. Log Hazard"  E_LOGHAZARD_S&s.*&&r_e_LogHazard_S&s.;
     %end;
	 %end;
   %end;
   %else %if (%elementinlist(&EventModel., GSPLINE GBSPLINE GNATURALCUBIC WSPLINE WBSPLINE WNATURALCUBIC RPNATURALCUBIC RPSPLINE RPBSPLINE)) and &event_nparam. > 1  %then %do;
   %if &JMStrataN. = 1 %then %do;
            %do i = 0 %to %eval(&event_nparam.-1);  
	          estimate "Event Process: Spline parameter &i." E_TIME_B&i.*&&r_e_time_b&i.; 
            %end;
   %end;
   %else %if &JMStrataN. > 1 %then %do;
     %do s=1 %to  &JMStrataN.;
       %do i = 0 %to %eval(&event_nparam.-1);
         estimate "Event Process: Stratum &s. Spline parameter &i." E_TIME_B&i._S&s.*&&r_e_time_b&i._s&s.; 
	   %end;
	 %end;
   %end;
   %end;
   %else %if &EventModel. = PIECEWISE %then %do; 
   %if &JMStrataN. = 1 %then %do;
       %do i = 1 %to %eval(&EventNknots.+1);
         estimate "Event Process: Log Hazard Period &i." E_LOGHAZARD_&i.*&&r_e_LogHazard_&i.;    
	   %end;
   %end;
   %else %if &JMStrataN. > 1 %then %do;
     %do s=1 %to  &JMStrataN.;
       %do i = 1 %to %eval(&EventNknots.+1);
         estimate "Event Process: Log Hazard Stratum &s. Period &i." E_LOGHAZARD_&i._S&s.*&&r_e_LogHazard_&i._s&s.;    
	   %end;
	 %end;
   %end;
   %end;

   %if &EventCovariates. NE %then %do;
    %do i = 1 %to &Number_EventCovariates;
     estimate "Event Process: &&EVENTCovariate_&i." &&EVENTCovsNLMIXED_ES_&i.;
	%end;
   %end;
   *if not (ONLYLONGI in AdditionalOptions);%end; 

   %if %elementinlist(CURRENT_VALUE, &SharedParam.)   %then %do;
    estimate 'Association (current-value)' &r_assoct_td.*ASSOCT_TD;
      %do i=1 %to &Number_SharedParamInt.;
	    estimate "Association by &&SharedParamInt_&i. (current-value)"  &&r_assoct_td_&i.*ASSOCT_TD_&i.;
      %end; 
   %end;
   %if %elementinlist(BASELINE_VALUE, &SharedParam.)   %then %do;
    estimate 'Association (baseline-value)' &r_assoct_bs.*ASSOCT_BS;
      %do i=1 %to &Number_SharedParamInt.;
	    estimate "Association by &&SharedParamInt_&i. (baseline-value)"  &&r_assoct_bs_&i.*ASSOCT_BS_&i.;
      %end; 
   %end;
      %if %elementinlist(CHANGE_FROM_BASELINE, &SharedParam.)   %then %do;
    estimate 'Association (change-from-baseline)' &r_assoct_ch.*ASSOCT_CH;
      %do i=1 %to &Number_SharedParamInt.;
	    estimate "Association by &&SharedParamInt_&i. (change-from-baseline)"  &&r_assoct_ch_&i.*ASSOCT_CH_&i.;
      %end; 
   %end;
   %if %elementinlist(SLOPE, &SharedParam.)  %then %do;
    estimate "Association (slope-dependent)" &r_assoct_sl.*ASSOCT_SL; 
	   %do i=1 %to &Number_SharedParamInt.;
	    estimate "Association by &&SharedParamInt_&i. (slope-dependent)"  &&r_assoct_sl_&i.*ASSOCT_SL_&i.;
      %end; 
   %end;
   %if %elementinlist(CUMULATIVE, &SharedParam.)  %then %do;
    estimate "Association (cumulative-dependent)" &r_assoct_cm.*ASSOCT_CM; 
	  %do i=1 %to &Number_SharedParamInt.;
	    estimate "Association by &&SharedParamInt_&i. (cumulative-dependent)"  &&r_assoct_cm_&i.*ASSOCT_CM_&i.;
      %end; 
   %end;

   %if %elementinlist(COEFFICIENTS, &SharedParam.) %then %do;
    %do i = 1 %to &Number_SharedCoefficients.; 
	%let what= &&Reffect&i.;
      estimate "Association (&what.-dependent)" &&r_assoct_re_&what.*ASSOCT_RE_&what.; 
	   %do j=1 %to &Number_SharedParamInt.;
	    estimate "Association by &&SharedParamInt_&j. (&what.-dependent)"  &&r_assoct_re_&what._&j.*ASSOCT_RE_&what._&j.;
      %end; 
	%end;
   %end;
   %if &OutputPredictions. NE  %then %do;
      %do i = 1 %to &Number_OutputPredictions;
         predict &&OutputPrediction_&i. out=&&OutputPredictData_&i.;
      %end;
   %end;

run;


%if %sysfunc(exist(_JMOutputParameters)) %then %do;
data &OutputParameters. (drop= gradient);
  set _JMOutputParameters;
  length LongiTimeModel $200. NLMixedOptions $200. AdditionalOptions $200.;
  LongiTimeModel="&LongiTimeModel.";
  NLMixedOptions="&NLMixedOptions.";
  AdditionalOptions="&AdditionalOptions.";
  rescaling = input(left(symget('R_'||parameter)),best16.);
  if abs(estimate)<0.1 or abs(estimate)>10 then do;
    put "NO" "TE: The rescaled estimate for " parameter " is not of a magnitude close to 1 (all rescaled estimates are expected to be around 1).";
    put "      It might be advisable to execute the macro again using the estimates from the current execution as initial parameters.";
   end;
  estimate=estimate*rescaling;

  * Several lines added/updated below in V2.01 to solve a bug;
  if rescaling>=0 then do;
    lower=lower*rescaling;
    upper=upper*rescaling;
  end;
  if rescaling<0 then do;
    lower2=upper*rescaling;
    upper=lower*rescaling;
	lower=lower2;
	drop lower2;
  end;
  GradientReScaled=Gradient;
  label GradientReScaled= "Gradient of the rescaled parameter" rescaling="Rescaling Factor for parameter";
  * Several lines added/updated above in V2.01 to solve a bug;

  StandardError=StandardError*abs(rescaling);

%if %elementinlist(CALCULATEEXECTIME, &AdditionalOptions.) %then %do;
  if _n_ = 1 then do;
  stopdate=date();
  stoptime=time();
  startdate= input(symget('startdate'), hex16.);
  starttime= input(symget('starttime'), hex16.);
  days=stopdate-startdate;
  if days=0 then seconds=stoptime-starttime;
  else seconds=days*24*60*60+stoptime-starttime;
  file print notitles; 
  message = "Execution Time (hours:minutes:seconds): "!!put(seconds,time.)!!".";
  put message;
  drop  stopdate stoptime startdate starttime days message;
  end;
%end;
run;


%if %elementinlist(FITSTATISTICS, &AdditionalOptions.) %then %do;
data &OutputParameters._Fit;
  set _JMFitStatistics; 
run;
%end;

%end;


%if NOT (%elementinlist(UNTOUCHEDLISTING, &AdditionalOptions.)) %then %do;
  ods listing close;
%end;

%mend jm;

